@@ -7,6 +7,7 @@ from typing import (
     Callable,
     Dict,
     List,
+    Mapping,
     NamedTuple,
     Optional,
     Set,
@@ -15,10 +16,7 @@ from typing import (
     Union,
 )
 
-from dagster import check
-from dagster.core.definitions.graph import GraphDefinition
-from dagster.core.definitions.input import InputDefinition, InputMapping
-from dagster.core.definitions.policy import RetryPolicy
+import dagster._check as check
 from dagster.core.errors import (
     DagsterInvalidDefinitionError,
     DagsterInvalidInvocationError,
@@ -27,7 +25,7 @@ from dagster.core.errors import (
 from dagster.utils import frozentags
 
 from .config import ConfigMapping
-from .decorators.solid import (
+from .decorators.solid_decorator import (
     DecoratedSolidFunction,
     NoContextDecoratedSolidFunction,
     resolve_checked_solid_fn_inputs,
@@ -37,22 +35,27 @@ from .dependency import (
     DynamicCollectDependencyDefinition,
     IDependencyDefinition,
     MultiDependencyDefinition,
-    SolidInvocation,
+    NodeInvocation,
 )
-from .hook import HookDefinition
+from .graph_definition import GraphDefinition
+from .hook_definition import HookDefinition
 from .inference import infer_output_props
-from .logger import LoggerDefinition
+from .input import InputDefinition, InputMapping
+from .logger_definition import LoggerDefinition
 from .output import OutputDefinition, OutputMapping
-from .resource import ResourceDefinition
-from .solid import NodeDefinition, SolidDefinition
+from .policy import RetryPolicy
+from .resource_definition import ResourceDefinition
+from .solid_definition import NodeDefinition, SolidDefinition
 from .utils import check_valid_name, validate_tags
 from .version_strategy import VersionStrategy
 
 if TYPE_CHECKING:
+    from dagster.core.execution.execute_in_process_result import ExecuteInProcessResult
     from dagster.core.instance import DagsterInstance
-    from .partition import PartitionedConfig
-    from .executor import ExecutorDefinition
-    from .job import JobDefinition
+
+    from .executor_definition import ExecutorDefinition
+    from .job_definition import JobDefinition
+    from .partition import PartitionedConfig, PartitionsDefinition
 
 
 _composition_stack: List["InProgressCompositionContext"] = []
@@ -68,7 +71,7 @@ def _not_invoked_warning(
     context_name: str,
 ) -> None:
     warning_message = (
-        "While in {context} context '{name}', received an uninvoked solid '{solid_name}'.\n"
+        "While in {context} context '{name}', received an uninvoked {node_type} '{solid_name}'.\n"
     )
     if solid.given_alias:
         warning_message += "'{solid_name}' was aliased as '{given_alias}'.\n"
@@ -84,6 +87,7 @@ def _not_invoked_warning(
         given_alias=solid.given_alias,
         tags=solid.tags,
         hooks=[hook.name for hook in solid.hook_defs],
+        node_type=solid.node_def.node_type_str,
     )
 
     warnings.warn(warning_message.strip())
@@ -191,7 +195,7 @@ class CompleteCompositionContext(NamedTuple):
 
     name: str
     solid_defs: List[NodeDefinition]
-    dependencies: Dict[Union[str, SolidInvocation], Dict[str, IDependencyDefinition]]
+    dependencies: Dict[Union[str, NodeInvocation], Dict[str, IDependencyDefinition]]
     input_mappings: List[InputMapping]
     output_mapping_dict: Dict[str, OutputMapping]
 
@@ -204,7 +208,7 @@ class CompleteCompositionContext(NamedTuple):
         pending_invocations: Dict[str, "PendingNodeInvocation"],
     ):
 
-        dep_dict: Dict[Union[str, SolidInvocation], Dict[str, IDependencyDefinition]] = {}
+        dep_dict: Dict[Union[str, NodeInvocation], Dict[str, IDependencyDefinition]] = {}
         node_def_dict: Dict[str, NodeDefinition] = {}
         input_mappings = []
 
@@ -257,7 +261,7 @@ class CompleteCompositionContext(NamedTuple):
                     check.failed(f"Unexpected input binding - got {node}")
 
             dep_dict[
-                SolidInvocation(
+                NodeInvocation(
                     invocation.node_def.name,
                     invocation.node_name,
                     tags=invocation.tags,
@@ -301,8 +305,8 @@ class PendingNodeInvocation:
             current_context().add_pending_invocation(self)
 
     def __call__(self, *args, **kwargs):
-        from .solid_invocation import solid_invocation_result
         from ..execution.context.invocation import UnboundSolidExecutionContext
+        from .solid_invocation import solid_invocation_result
 
         node_name = self.given_alias if self.given_alias else self.node_def.name
 
@@ -560,13 +564,16 @@ class PendingNodeInvocation:
         self,
         name: Optional[str] = None,
         description: Optional[str] = None,
-        resource_defs: Optional[Dict[str, ResourceDefinition]] = None,
-        config: Union[ConfigMapping, Dict[str, Any], "PartitionedConfig"] = None,
+        resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
+        config: Optional[Union[ConfigMapping, Dict[str, Any], "PartitionedConfig"]] = None,
         tags: Optional[Dict[str, Any]] = None,
-        logger_defs: Optional[Dict[str, LoggerDefinition]] = None,
+        logger_defs: Optional[Mapping[str, LoggerDefinition]] = None,
         executor_def: Optional["ExecutorDefinition"] = None,
         hooks: Optional[AbstractSet[HookDefinition]] = None,
+        op_retry_policy: Optional[RetryPolicy] = None,
         version_strategy: Optional[VersionStrategy] = None,
+        partitions_def: Optional["PartitionsDefinition"] = None,
+        input_values: Optional[Mapping[str, object]] = None,
     ) -> "JobDefinition":
         if not isinstance(self.node_def, GraphDefinition):
             raise DagsterInvalidInvocationError(
@@ -576,6 +583,8 @@ class PendingNodeInvocation:
 
         tags = check.opt_dict_param(tags, "tags", key_type=str)
         hooks = check.opt_set_param(hooks, "hooks", HookDefinition)
+        input_values = check.opt_mapping_param(input_values, "input_values")
+        op_retry_policy = check.opt_inst_param(op_retry_policy, "op_retry_policy", RetryPolicy)
         job_hooks: Set[HookDefinition] = set()
         job_hooks.update(check.opt_set_param(hooks, "hooks", HookDefinition))
         job_hooks.update(self.hook_defs)
@@ -588,16 +597,21 @@ class PendingNodeInvocation:
             logger_defs=logger_defs,
             executor_def=executor_def,
             hooks=job_hooks,
+            op_retry_policy=op_retry_policy,
             version_strategy=version_strategy,
+            partitions_def=partitions_def,
+            input_values=input_values,
         )
 
     def execute_in_process(
         self,
-        run_config: Any = None,
+        run_config: Optional[Any] = None,
         instance: Optional["DagsterInstance"] = None,
         resources: Optional[Dict[str, Any]] = None,
         raise_on_error: bool = True,
-    ):
+        run_id: Optional[str] = None,
+        input_values: Optional[Mapping[str, object]] = None,
+    ) -> "ExecuteInProcessResult":
         if not isinstance(self.node_def, GraphDefinition):
             raise DagsterInvalidInvocationError(
                 "Attemped to call `execute_in_process` on a composite solid.  Only graphs "
@@ -606,25 +620,21 @@ class PendingNodeInvocation:
 
         from dagster.core.execution.build_resources import wrap_resources_for_execution
         from dagster.core.execution.execute_in_process import core_execute_in_process
-        from .mode import ModeDefinition
-        from .job import JobDefinition
-        from .executor import execute_in_process_executor
 
-        if len(self.node_def.input_defs) > 0:
-            raise DagsterInvariantViolationError(
-                "Graphs with inputs cannot be used with execute_in_process at this time."
-            )
+        from .executor_definition import execute_in_process_executor
+        from .job_definition import JobDefinition
+
+        input_values = check.opt_mapping_param(input_values, "input_values")
 
         ephemeral_job = JobDefinition(
             name=self.given_alias,
             graph_def=self.node_def,
-            mode_def=ModeDefinition(
-                executor_defs=[execute_in_process_executor],
-                resource_defs=wrap_resources_for_execution(resources),
-            ),
+            executor_def=execute_in_process_executor,
+            resource_defs=wrap_resources_for_execution(resources),
             tags=self.tags,
             hook_defs=self.hook_defs,
-            solid_retry_policy=self.retry_policy,
+            op_retry_policy=self.retry_policy,
+            _input_values=input_values,
         )
 
         return core_execute_in_process(
@@ -634,6 +644,7 @@ class PendingNodeInvocation:
             instance=instance,
             output_capturing_enabled=True,
             raise_on_error=raise_on_error,
+            run_id=run_id,
         )
 
 
@@ -659,7 +670,7 @@ class InvokedSolidOutputHandle:
     def __iter__(self):
         raise DagsterInvariantViolationError(
             'Attempted to iterate over an {cls}. This object represents the output "{out}" '
-            'from the solid "{solid}". Consider yielding multiple Outputs if you seek to pass '
+            'from the solid "{solid}". Consider defining multiple Outs if you seek to pass '
             "different parts of this output to different solids.".format(
                 cls=self.__class__.__name__, out=self.output_name, solid=self.solid_name
             )
@@ -668,7 +679,7 @@ class InvokedSolidOutputHandle:
     def __getitem__(self, idx):
         raise DagsterInvariantViolationError(
             'Attempted to index in to an {cls}. This object represents the output "{out}" '
-            "from the {described_node}. Consider yielding multiple Outputs if you seek to pass "
+            "from the {described_node}. Consider defining multiple Outs if you seek to pass "
             "different parts of this output to different {node_type}s.".format(
                 cls=self.__class__.__name__,
                 out=self.output_name,
@@ -945,7 +956,7 @@ def do_composition(
 ) -> Tuple[
     List[InputMapping],
     List[OutputMapping],
-    Dict[Union[str, SolidInvocation], Dict[str, IDependencyDefinition]],
+    Dict[Union[str, NodeInvocation], Dict[str, IDependencyDefinition]],
     List[NodeDefinition],
     Optional[ConfigMapping],
     List[str],
@@ -989,7 +1000,6 @@ def do_composition(
         fn_name=graph_name,
         compute_fn=compute_fn,
         explicit_input_defs=provided_input_defs,
-        context_required=False,
         exclude_nothing=False,
     )
 
@@ -1033,10 +1043,17 @@ def do_composition(
         ]
 
         if len(mappings) == 0:
+            if decorator_name in {"@op", "@graph"}:
+                invocation_name = "op/graph"
+            else:
+                invocation_name = "solid"
             raise DagsterInvalidDefinitionError(
                 "{decorator_name} '{graph_name}' has unmapped input '{input_name}'. "
-                "Remove it or pass it to the appropriate solid invocation.".format(
-                    decorator_name=decorator_name, graph_name=graph_name, input_name=defn.name
+                "Remove it or pass it to the appropriate {invocation_name} invocation.".format(
+                    decorator_name=decorator_name,
+                    graph_name=graph_name,
+                    input_name=defn.name,
+                    invocation_name=invocation_name,
                 )
             )
 
