@@ -1,11 +1,12 @@
 from abc import ABC, abstractmethod
-from collections import defaultdict, namedtuple
+from collections import defaultdict
 from enum import Enum
 from typing import (
     TYPE_CHECKING,
     AbstractSet,
     Any,
     Dict,
+    Iterator,
     List,
     NamedTuple,
     Optional,
@@ -15,25 +16,43 @@ from typing import (
     cast,
 )
 
-from dagster import check
+import dagster._check as check
 from dagster.core.definitions.policy import RetryPolicy
 from dagster.core.errors import DagsterInvalidDefinitionError
-from dagster.serdes import whitelist_for_serdes
+from dagster.serdes.serdes import (
+    DefaultNamedTupleSerializer,
+    WhitelistMap,
+    register_serdes_tuple_fallbacks,
+    whitelist_for_serdes,
+)
 from dagster.utils import frozentags
 
-from .hook import HookDefinition
+from .hook_definition import HookDefinition
 from .input import FanInInputPointer, InputDefinition, InputMapping, InputPointer
 from .output import OutputDefinition
 from .utils import DEFAULT_OUTPUT, struct_to_string, validate_tags
 
 if TYPE_CHECKING:
+    from .asset_layer import AssetLayer
     from .composition import MappedInputPlaceholder
-    from .graph import GraphDefinition
-    from .solid import NodeDefinition
+    from .graph_definition import GraphDefinition
+    from .node_definition import NodeDefinition
+    from .resource_requirement import ResourceRequirement
 
 
-class SolidInvocation(namedtuple("Solid", "name alias tags hook_defs retry_policy")):
-    """Identifies an instance of a solid in a pipeline dependency structure.
+class NodeInvocation(
+    NamedTuple(
+        "Node",
+        [
+            ("name", str),
+            ("alias", Optional[str]),
+            ("tags", Dict[str, Any]),
+            ("hook_defs", AbstractSet[HookDefinition]),
+            ("retry_policy", Optional[RetryPolicy]),
+        ],
+    )
+):
+    """Identifies an instance of a node in a graph dependency structure.
 
     Args:
         name (str): Name of the solid of which this is an instance.
@@ -46,30 +65,18 @@ class SolidInvocation(namedtuple("Solid", "name alias tags hook_defs retry_polic
 
     Examples:
 
-        .. code-block:: python
-
-            pipeline = PipelineDefinition(
-                solid_defs=[solid_1, solid_2]
-                dependencies={
-                    SolidInvocation('solid_1', alias='other_name') : {
-                        'input_name' : DependencyDefinition('solid_1'),
-                    },
-                    'solid_2' : {
-                        'input_name': DependencyDefinition('other_name'),
-                    },
-                }
-            )
-
     In general, users should prefer not to construct this class directly or use the
-    :py:class:`PipelineDefinition` API that requires instances of this class. Instead, use the
-    :py:func:`@pipeline <pipeline>` API:
+    :py:class:`JobDefinition` API that requires instances of this class. Instead, use the
+    :py:func:`@job <job>` API:
 
     .. code-block:: python
 
-        @pipeline
-        def pipeline():
-            other_name = solid_1.alias('other_name')
-            solid_2(other_name(solid_1))
+        from dagster import job
+
+        @job
+        def my_job():
+            other_name = some_op.alias('other_name')
+            some_graph(other_name(some_op))
 
     """
 
@@ -77,8 +84,8 @@ class SolidInvocation(namedtuple("Solid", "name alias tags hook_defs retry_polic
         cls,
         name: str,
         alias: Optional[str] = None,
-        tags: Dict[str, str] = None,
-        hook_defs: AbstractSet[HookDefinition] = None,
+        tags: Optional[Dict[str, str]] = None,
+        hook_defs: Optional[AbstractSet[HookDefinition]] = None,
         retry_policy: Optional[RetryPolicy] = None,
     ):
         return super().__new__(
@@ -93,15 +100,12 @@ class SolidInvocation(namedtuple("Solid", "name alias tags hook_defs retry_polic
         )
 
 
-class Solid:
-    """
-    Solid invocation within a pipeline. Defined by its name inside the pipeline.
+SolidInvocation = NodeInvocation
 
-    Attributes:
-        name (str):
-            Name of the solid inside the pipeline. Must be unique per-pipeline.
-        definition (NodeDefinition):
-            Definition of the Node.
+
+class Node:
+    """
+    Node invocation within a graph. Identified by its name inside the graph.
     """
 
     def __init__(
@@ -109,12 +113,12 @@ class Solid:
         name: str,
         definition: "NodeDefinition",
         graph_definition: "GraphDefinition",
-        tags: Dict[str, str] = None,
+        tags: Optional[Dict[str, str]] = None,
         hook_defs: Optional[AbstractSet[HookDefinition]] = None,
         retry_policy: Optional[RetryPolicy] = None,
     ):
-        from .graph import GraphDefinition
-        from .solid import NodeDefinition
+        from .graph_definition import GraphDefinition
+        from .solid_definition import NodeDefinition
 
         self.name = check.str_param(name, "name")
         self.definition = check.inst_param(definition, "definition", NodeDefinition)
@@ -166,11 +170,24 @@ class Solid:
         return self.definition.output_def_named(name)
 
     @property
-    def is_composite(self) -> bool:
-        from .graph import GraphDefinition
+    def is_graph(self) -> bool:
+        from .graph_definition import GraphDefinition
 
         return isinstance(self.definition, GraphDefinition)
 
+    def describe_node(self) -> str:
+        from .op_definition import OpDefinition
+        from .solid_definition import CompositeSolidDefinition, SolidDefinition
+
+        if isinstance(self.definition, CompositeSolidDefinition):
+            return f"composite solid '{self.name}'"
+        elif isinstance(self.definition, OpDefinition):
+            return f"op '{self.name}'"
+        elif isinstance(self.definition, SolidDefinition):
+            return f"solid '{self.name}'"
+        else:
+            return f"graph '{self.name}'"
+
     @property
     def input_dict(self):
         return self.definition.input_dict
@@ -190,7 +207,14 @@ class Solid:
         )
 
     def container_mapped_input(self, input_name: str) -> InputMapping:
-        return self.graph_definition.input_mapping_for_pointer(InputPointer(self.name, input_name))
+        mapping = self.graph_definition.input_mapping_for_pointer(
+            InputPointer(self.name, input_name)
+        )
+        if mapping is None:
+            check.failed(
+                f"container does not map input {input_name}, check container_maps_input first"
+            )
+        return mapping
 
     def container_maps_fan_in_input(self, input_name: str, fan_in_index: int) -> bool:
         return (
@@ -201,9 +225,16 @@ class Solid:
         )
 
     def container_mapped_fan_in_input(self, input_name: str, fan_in_index: int) -> InputMapping:
-        return self.graph_definition.input_mapping_for_pointer(
+        mapping = self.graph_definition.input_mapping_for_pointer(
             FanInInputPointer(self.name, input_name, fan_in_index)
         )
+        if mapping is None:
+            check.failed(
+                f"container does not map fan-in {input_name} idx {fan_in_index}, check "
+                "container_maps_fan_in_input first"
+            )
+
+        return mapping
 
     @property
     def hook_defs(self) -> AbstractSet[HookDefinition]:
@@ -213,14 +244,72 @@ class Solid:
     def retry_policy(self) -> Optional[RetryPolicy]:
         return self._retry_policy
 
+    def get_resource_requirements(
+        self,
+        outer_container: "GraphDefinition",
+        parent_handle: Optional["NodeHandle"] = None,
+        asset_layer: Optional["AssetLayer"] = None,
+    ) -> Iterator["ResourceRequirement"]:
+        from .resource_requirement import InputManagerRequirement
 
-@whitelist_for_serdes
-class SolidHandle(namedtuple("_SolidHandle", "name parent")):
-    def __new__(cls, name: str, parent: Optional["SolidHandle"]):
-        return super(SolidHandle, cls).__new__(
+        cur_node_handle = NodeHandle(self.name, parent_handle)
+
+        if not self.is_graph:
+            solid_def = self.definition.ensure_solid_def()
+            for requirement in solid_def.get_resource_requirements((cur_node_handle, asset_layer)):
+                # If requirement is a root input manager requirement, but the corresponding node has an upstream output, then ignore the requirement.
+                if isinstance(
+                    requirement, InputManagerRequirement
+                ) and outer_container.dependency_structure.has_deps(
+                    SolidInputHandle(self, solid_def.input_def_named(requirement.input_name))
+                ):
+                    continue
+                yield requirement
+            for hook_def in self.hook_defs:
+                yield from hook_def.get_resource_requirements(self.describe_node())
+        else:
+            graph_def = self.definition.ensure_graph_def()
+            for node in graph_def.node_dict.values():
+                yield from node.get_resource_requirements(
+                    asset_layer=asset_layer,
+                    outer_container=graph_def,
+                    parent_handle=cur_node_handle,
+                )
+
+
+class NodeHandleSerializer(DefaultNamedTupleSerializer):
+    @classmethod
+    def value_to_storage_dict(
+        cls,
+        value: NamedTuple,
+        whitelist_map: WhitelistMap,
+        descent_path: str,
+    ) -> Dict[str, Any]:
+        storage = super().value_to_storage_dict(
+            value,
+            whitelist_map,
+            descent_path,
+        )
+        # persist using legacy name SolidHandle
+        storage["__class__"] = "SolidHandle"
+        return storage
+
+
+@whitelist_for_serdes(serializer=NodeHandleSerializer)
+class NodeHandle(
+    # mypy does not yet support recursive types
+    # NamedTuple("_NodeHandle", [("name", str), ("parent", Optional["NodeHandle"])])
+    NamedTuple("_NodeHandle", [("name", str), ("parent", Any)])
+):
+    """
+    A structured object to identify nodes in the potentially recursive graph structure.
+    """
+
+    def __new__(cls, name: str, parent: Optional["NodeHandle"]):
+        return super(NodeHandle, cls).__new__(
             cls,
             check.str_param(name, "name"),
-            check.opt_inst_param(parent, "parent", SolidHandle),
+            check.opt_inst_param(parent, "parent", NodeHandle),
         )
 
     def __str__(self):
@@ -230,7 +319,7 @@ class SolidHandle(namedtuple("_SolidHandle", "name parent")):
     def path(self) -> List[str]:
         """Return a list representation of the handle.
 
-        Inverse of SolidHandle.from_path.
+        Inverse of NodeHandle.from_path.
 
         Returns:
             List[str]:
@@ -246,20 +335,20 @@ class SolidHandle(namedtuple("_SolidHandle", "name parent")):
     def to_string(self) -> str:
         """Return a unique string representation of the handle.
 
-        Inverse of SolidHandle.from_string.
+        Inverse of NodeHandle.from_string.
         """
         return self.parent.to_string() + "." + self.name if self.parent else self.name
 
-    def is_or_descends_from(self, handle: "SolidHandle") -> bool:
+    def is_or_descends_from(self, handle: "NodeHandle") -> bool:
         """Check if the handle is or descends from another handle.
 
         Args:
-            handle (SolidHandle): The handle to check against.
+            handle (NodeHandle): The handle to check against.
 
         Returns:
             bool:
         """
-        check.inst_param(handle, "handle", SolidHandle)
+        check.inst_param(handle, "handle", NodeHandle)
 
         for idx in range(len(handle.path)):
             if idx >= len(self.path):
@@ -268,25 +357,25 @@ class SolidHandle(namedtuple("_SolidHandle", "name parent")):
                 return False
         return True
 
-    def pop(self, ancestor: "SolidHandle") -> Optional["SolidHandle"]:
+    def pop(self, ancestor: "NodeHandle") -> Optional["NodeHandle"]:
         """Return a copy of the handle with some of its ancestors pruned.
 
         Args:
-            ancestor (SolidHandle): Handle to an ancestor of the current handle.
+            ancestor (NodeHandle): Handle to an ancestor of the current handle.
 
         Returns:
-            SolidHandle:
+            NodeHandle:
 
         Example:
 
         .. code-block:: python
 
-            handle = SolidHandle('baz', SolidHandle('bar', SolidHandle('foo', None)))
-            ancestor = SolidHandle('bar', SolidHandle('foo', None))
-            assert handle.pop(ancestor) == SolidHandle('baz', None)
+            handle = NodeHandle('baz', NodeHandle('bar', NodeHandle('foo', None)))
+            ancestor = NodeHandle('bar', NodeHandle('foo', None))
+            assert handle.pop(ancestor) == NodeHandle('baz', None)
         """
 
-        check.inst_param(ancestor, "ancestor", SolidHandle)
+        check.inst_param(ancestor, "ancestor", NodeHandle)
         check.invariant(
             self.is_or_descends_from(ancestor),
             "Handle {handle} does not descend from {ancestor}".format(
@@ -294,76 +383,102 @@ class SolidHandle(namedtuple("_SolidHandle", "name parent")):
             ),
         )
 
-        return SolidHandle.from_path(self.path[len(ancestor.path) :])
+        return NodeHandle.from_path(self.path[len(ancestor.path) :])
 
-    def with_ancestor(self, ancestor: "SolidHandle") -> Optional["SolidHandle"]:
+    def with_ancestor(self, ancestor: "NodeHandle") -> Optional["NodeHandle"]:
         """Returns a copy of the handle with an ancestor grafted on.
 
         Args:
-            ancestor (SolidHandle): Handle to the new ancestor.
+            ancestor (NodeHandle): Handle to the new ancestor.
 
         Returns:
-            SolidHandle:
+            NodeHandle:
 
         Example:
 
         .. code-block:: python
 
-            handle = SolidHandle('baz', SolidHandle('bar', SolidHandle('foo', None)))
-            ancestor = SolidHandle('quux' None)
-            assert handle.with_ancestor(ancestor) == SolidHandle(
-                'baz', SolidHandle('bar', SolidHandle('foo', SolidHandle('quux', None)))
+            handle = NodeHandle('baz', NodeHandle('bar', NodeHandle('foo', None)))
+            ancestor = NodeHandle('quux' None)
+            assert handle.with_ancestor(ancestor) == NodeHandle(
+                'baz', NodeHandle('bar', NodeHandle('foo', NodeHandle('quux', None)))
             )
         """
-        check.opt_inst_param(ancestor, "ancestor", SolidHandle)
+        check.opt_inst_param(ancestor, "ancestor", NodeHandle)
 
-        return SolidHandle.from_path((ancestor.path if ancestor else []) + self.path)
+        return NodeHandle.from_path((ancestor.path if ancestor else []) + self.path)
 
     @staticmethod
-    def from_path(path: List[str]) -> Optional["SolidHandle"]:
+    def from_path(path: List[str]) -> "NodeHandle":
         check.list_param(path, "path", of_type=str)
 
-        cur = None
+        cur: Optional["NodeHandle"] = None
         while len(path) > 0:
-            cur = SolidHandle(name=path.pop(0), parent=cur)
+            cur = NodeHandle(name=path.pop(0), parent=cur)
+
+        if cur is None:
+            check.failed(f"Invalid handle path {path}")
+
         return cur
 
     @staticmethod
-    def from_string(handle_str: str) -> Optional["SolidHandle"]:
+    def from_string(handle_str: str) -> "NodeHandle":
         check.str_param(handle_str, "handle_str")
 
         path = handle_str.split(".")
-        return SolidHandle.from_path(path)
+        return NodeHandle.from_path(path)
 
     @classmethod
-    def from_dict(cls, dict_repr: Dict[str, Any]) -> Optional["SolidHandle"]:
-        """This method makes it possible to load a potentially nested SolidHandle after a
-        roundtrip through json.loads(json.dumps(SolidHandle._asdict()))"""
+    def from_dict(cls, dict_repr: Dict[str, Any]) -> Optional["NodeHandle"]:
+        """This method makes it possible to load a potentially nested NodeHandle after a
+        roundtrip through json.loads(json.dumps(NodeHandle._asdict()))"""
 
         check.dict_param(dict_repr, "dict_repr", key_type=str)
         check.invariant(
-            "name" in dict_repr, "Dict representation of SolidHandle must have a 'name' key"
+            "name" in dict_repr, "Dict representation of NodeHandle must have a 'name' key"
         )
         check.invariant(
-            "parent" in dict_repr, "Dict representation of SolidHandle must have a 'parent' key"
+            "parent" in dict_repr, "Dict representation of NodeHandle must have a 'parent' key"
         )
 
         if isinstance(dict_repr["parent"], (list, tuple)):
-            dict_repr["parent"] = SolidHandle.from_dict(
+            dict_repr["parent"] = NodeHandle.from_dict(
                 {
                     "name": dict_repr["parent"][0],
                     "parent": dict_repr["parent"][1],
                 }
             )
 
-        return SolidHandle(**{k: dict_repr[k] for k in ["name", "parent"]})
+        return NodeHandle(**{k: dict_repr[k] for k in ["name", "parent"]})
 
 
-class SolidInputHandle(namedtuple("_SolidInputHandle", "solid input_def")):
-    def __new__(cls, solid: Solid, input_def: InputDefinition):
+class NodeInputHandle(
+    NamedTuple("_NodeInputHandle", [("node_handle", NodeHandle), ("input_name", str)])
+):
+    """
+    A structured object to uniquely identify inputs in the potentially recursive graph structure.
+    """
+
+
+class NodeOutputHandle(
+    NamedTuple("_NodeOutputHandle", [("node_handle", NodeHandle), ("output_name", str)])
+):
+    """
+    A structured object to uniquely identify outputs in the potentially recursive graph structure.
+    """
+
+
+# previous name for NodeHandle was SolidHandle
+register_serdes_tuple_fallbacks({"SolidHandle": NodeHandle})
+
+
+class SolidInputHandle(
+    NamedTuple("_SolidInputHandle", [("solid", Node), ("input_def", InputDefinition)])
+):
+    def __new__(cls, solid: Node, input_def: InputDefinition):
         return super(SolidInputHandle, cls).__new__(
             cls,
-            check.inst_param(solid, "solid", Solid),
+            check.inst_param(solid, "solid", Node),
             check.inst_param(input_def, "input_def", InputDefinition),
         )
 
@@ -395,11 +510,13 @@ class SolidInputHandle(namedtuple("_SolidInputHandle", "solid input_def")):
         return self.input_def.name
 
 
-class SolidOutputHandle(namedtuple("_SolidOutputHandle", "solid output_def")):
-    def __new__(cls, solid: Solid, output_def: OutputDefinition):
+class SolidOutputHandle(
+    NamedTuple("_SolidOutputHandle", [("solid", Node), ("output_def", OutputDefinition)])
+):
+    def __new__(cls, solid: Node, output_def: OutputDefinition):
         return super(SolidOutputHandle, cls).__new__(
             cls,
-            check.inst_param(solid, "solid", Solid),
+            check.inst_param(solid, "solid", Node),
             check.inst_param(output_def, "output_def", OutputDefinition),
         )
 
@@ -451,47 +568,72 @@ class IDependencyDefinition(ABC):  # pylint: disable=no-init
 
 
 class DependencyDefinition(
-    namedtuple("_DependencyDefinition", "solid output description"), IDependencyDefinition
+    NamedTuple(
+        "_DependencyDefinition", [("node", str), ("output", str), ("description", Optional[str])]
+    ),
+    IDependencyDefinition,
 ):
-    """Represents an edge in the DAG of solid instances forming a pipeline.
+    """Represents an edge in the DAG of nodes (ops or graphs) forming a job.
 
     This object is used at the leaves of a dictionary structure that represents the complete
-    dependency structure of a pipeline whose keys represent the dependent solid and dependent
+    dependency structure of a job whose keys represent the dependent node and dependent
     input, so this object only contains information about the dependee.
 
-    Concretely, if the input named 'input' of solid_b depends on the output named 'result' of
-    solid_a, this structure will look as follows:
+    Concretely, if the input named 'input' of op_b depends on the output named 'result' of
+    op_a, and the output named 'other_result' of graph_a, the structure will look as follows:
 
     .. code-block:: python
 
         dependency_structure = {
-            'solid_b': {
-                'input': DependencyDefinition('solid_a', 'result')
+            'my_downstream_op': {
+                'input': DependencyDefinition('my_upstream_op', 'result')
+            }
+            'my_downstream_op': {
+                'input': DependencyDefinition('my_upstream_graph', 'result')
             }
         }
 
     In general, users should prefer not to construct this class directly or use the
-    :py:class:`PipelineDefinition` API that requires instances of this class. Instead, use the
-    :py:func:`@pipeline <pipeline>` API:
+    :py:class:`JobDefinition` API that requires instances of this class. Instead, use the
+    :py:func:`@job <job>` API:
 
     .. code-block:: python
 
-        @pipeline
-        def pipeline():
-            solid_b(solid_a())
+        @job
+        def the_job():
+            node_b(node_a())
 
 
     Args:
-        solid (str): The name of the solid that is depended on, that is, from which the value
-            passed between the two solids originates.
+        solid (str): (legacy) The name of the solid that is depended on, that is, from which the value
+            passed between the two nodes originates.
         output (Optional[str]): The name of the output that is depended on. (default: "result")
         description (Optional[str]): Human-readable description of this dependency.
+        node (str): The name of the node (op or graph) that is depended on, that is, from which the value
+            passed between the two nodes originates.
     """
 
-    def __new__(cls, solid: str, output: str = DEFAULT_OUTPUT, description: Optional[str] = None):
+    def __new__(
+        cls,
+        solid: Optional[str] = None,
+        output: str = DEFAULT_OUTPUT,
+        description: Optional[str] = None,
+        node: Optional[str] = None,
+    ):
+        if solid and node:
+            raise DagsterInvalidDefinitionError(
+                "Both ``node`` and legacy ``solid`` arguments provided to DependencyDefinition. Please use one or the other."
+            )
+
+        if not solid and not node:
+            raise DagsterInvalidDefinitionError(
+                "Expected node parameter to be str for DependencyDefinition"
+            )
+
+        node = node or solid
         return super(DependencyDefinition, cls).__new__(
             cls,
-            check.str_param(solid, "solid"),
+            check.str_param(node, "node"),
             check.str_param(output, "output"),
             check.opt_str_param(description, "description"),
         )
@@ -502,50 +644,59 @@ class DependencyDefinition(
     def is_fan_in(self) -> bool:
         return False
 
+    @property
+    def solid(self) -> str:
+        return self.node
+
+    def get_op_dependencies(self) -> List["DependencyDefinition"]:
+        return [self]
+
 
 class MultiDependencyDefinition(
-    namedtuple("_MultiDependencyDefinition", "dependencies"), IDependencyDefinition
+    NamedTuple(
+        "_MultiDependencyDefinition",
+        [("dependencies", List[Union[DependencyDefinition, Type["MappedInputPlaceholder"]]])],
+    ),
+    IDependencyDefinition,
 ):
-    """Represents a fan-in edge in the DAG of solid instances forming a pipeline.
+    """Represents a fan-in edge in the DAG of op instances forming a job.
 
     This object is used only when an input of type ``List[T]`` is assembled by fanning-in multiple
     upstream outputs of type ``T``.
 
     This object is used at the leaves of a dictionary structure that represents the complete
-    dependency structure of a pipeline whose keys represent the dependent solid and dependent
+    dependency structure of a job or pipeline whose keys represent the dependent ops or graphs and dependent
     input, so this object only contains information about the dependee.
 
-    Concretely, if the input named 'input' of solid_c depends on the outputs named 'result' of
-    solid_a and solid_b, this structure will look as follows:
+    Concretely, if the input named 'input' of op_c depends on the outputs named 'result' of
+    op_a and op_b, this structure will look as follows:
 
     .. code-block:: python
 
         dependency_structure = {
-            'solid_c': {
+            'op_c': {
                 'input': MultiDependencyDefinition(
                     [
-                        DependencyDefinition('solid_a', 'result'),
-                        DependencyDefinition('solid_b', 'result')
+                        DependencyDefinition('op_a', 'result'),
+                        DependencyDefinition('op_b', 'result')
                     ]
                 )
             }
         }
 
     In general, users should prefer not to construct this class directly or use the
-    :py:class:`PipelineDefinition` API that requires instances of this class. Instead, use the
-    :py:func:`@pipeline <pipeline>` API:
+    :py:class:`JobDefinition` API that requires instances of this class. Instead, use the
+    :py:func:`@job <job>` API:
 
     .. code-block:: python
 
-        @pipeline
-        def pipeline():
-            solid_c(solid_a(), solid_b())
+        @job
+        def the_job():
+            op_c(op_a(), op_b())
 
     Args:
-        solid (str): The name of the solid that is depended on, that is, from which the value
-            passed between the two solids originates.
-        output (Optional[str]): The name of the output that is depended on. (default: "result")
-        description (Optional[str]): Human-readable description of this dependency.
+        dependencies (List[Union[DependencyDefinition, Type[MappedInputPlaceHolder]]]): List of
+            upstream dependencies fanned in to this input.
     """
 
     def __new__(
@@ -561,7 +712,7 @@ class MultiDependencyDefinition(
                 key = dep.solid + ":" + dep.output
                 if key in seen:
                     raise DagsterInvalidDefinitionError(
-                        'Duplicate dependencies on solid "{dep.solid}" output "{dep.output}" '
+                        'Duplicate dependencies on node "{dep.solid}" output "{dep.output}" '
                         "used in the same MultiDependencyDefinition.".format(dep=dep)
                     )
                 seen[key] = True
@@ -575,6 +726,9 @@ class MultiDependencyDefinition(
     def get_solid_dependencies(self) -> List[DependencyDefinition]:
         return [dep for dep in self.dependencies if isinstance(dep, DependencyDefinition)]
 
+    def get_node_dependencies(self) -> List[DependencyDefinition]:
+        return self.get_solid_dependencies()
+
     def is_fan_in(self) -> bool:
         return True
 
@@ -602,12 +756,12 @@ InputToOutputHandleDict = Dict[SolidInputHandle, DepTypeAndOutputHandles]
 
 
 def _create_handle_dict(
-    solid_dict: Dict[str, Solid],
+    solid_dict: Dict[str, Node],
     dep_dict: Dict[str, Dict[str, IDependencyDefinition]],
 ) -> InputToOutputHandleDict:
     from .composition import MappedInputPlaceholder
 
-    check.dict_param(solid_dict, "solid_dict", key_type=str, value_type=Solid)
+    check.dict_param(solid_dict, "solid_dict", key_type=str, value_type=Node)
     check.two_dim_dict_param(dep_dict, "dep_dict", value_type=IDependencyDefinition)
 
     handle_dict: InputToOutputHandleDict = {}
@@ -650,7 +804,7 @@ def _create_handle_dict(
 
 class DependencyStructure:
     @staticmethod
-    def from_definitions(solids: Dict[str, Solid], dep_dict: Dict[str, Any]):
+    def from_definitions(solids: Dict[str, Node], dep_dict: Dict[str, Any]):
         return DependencyStructure(list(dep_dict.keys()), _create_handle_dict(solids, dep_dict))
 
     def __init__(self, solid_names: List[str], handle_dict: InputToOutputHandleDict):
@@ -667,7 +821,7 @@ class DependencyStructure:
         # solid_name => output_handle => list[input_handle]
         self._solid_output_index: dict = defaultdict(lambda: defaultdict(list))
 
-        # solid_name => dynamic output_handle that this will solid dupe for
+        # solid_name => dynamic output_handle that this solid will dupe for
         self._dynamic_fan_out_index: dict = {}
 
         # solid_name => set of dynamic output_handle this collects over
@@ -717,7 +871,9 @@ class DependencyStructure:
                         self._dynamic_fan_out_index[output_handle.solid_name],
                     )
                 else:
-                    check.failed("Unexpected dynamic fan in dep created")
+                    check.failed(
+                        f"Unexpected dynamic fan in dep created {output_handle} -> {input_handle}"
+                    )
 
                 output_handle_list = [output_handle]
             else:
@@ -738,15 +894,15 @@ class DependencyStructure:
             input_handle.input_name
         ):
             raise DagsterInvalidDefinitionError(
-                f'Solid "{input_handle.solid_name}" cannot be downstream of dynamic output '
-                f'"{output_handle.describe()}" since input "{input_handle.input_name}" maps to a solid '
-                "that is already downstream of another dynamic output. Solids cannot be downstream of more "
+                f"{input_handle.solid.describe_node()} cannot be downstream of dynamic output "
+                f'"{output_handle.describe()}" since input "{input_handle.input_name}" maps to a node '
+                "that is already downstream of another dynamic output. Nodes cannot be downstream of more "
                 "than one dynamic output"
             )
 
         if self._collect_index.get(input_handle.solid_name):
             raise DagsterInvalidDefinitionError(
-                f'Solid "{input_handle.solid_name}" cannot be both downstream of dynamic output '
+                f"{input_handle.solid.describe_node()} cannot be both downstream of dynamic output "
                 f"{output_handle.describe()} and collect over dynamic output "
                 f"{list(self._collect_index[input_handle.solid_name])[0].describe()}."
             )
@@ -757,7 +913,7 @@ class DependencyStructure:
 
         if self._dynamic_fan_out_index[input_handle.solid_name] != output_handle:
             raise DagsterInvalidDefinitionError(
-                f'Solid "{input_handle.solid_name}" cannot be downstream of more than one dynamic output. '
+                f"{input_handle.solid.describe_node()} cannot be downstream of more than one dynamic output. "
                 f'It is downstream of both "{output_handle.describe()}" and '
                 f'"{self._dynamic_fan_out_index[input_handle.solid_name].describe()}"'
             )
@@ -766,10 +922,10 @@ class DependencyStructure:
         self,
         input_handle: SolidInputHandle,
         output_handle: SolidOutputHandle,
-    ):
+    ) -> None:
         if self._dynamic_fan_out_index.get(input_handle.solid_name):
             raise DagsterInvalidDefinitionError(
-                f'Solid "{input_handle.solid_name}" cannot both collect over dynamic output '
+                f"{input_handle.solid.describe_node()} cannot both collect over dynamic output "
                 f"{output_handle.describe()} and be downstream of the dynamic output "
                 f"{self._dynamic_fan_out_index[input_handle.solid_name].describe()}."
             )
@@ -779,7 +935,7 @@ class DependencyStructure:
         # if the output is already fanned out
         if self._dynamic_fan_out_index.get(output_handle.solid_name):
             raise DagsterInvalidDefinitionError(
-                f'Solid "{input_handle.solid_name}" cannot be downstream of more than one dynamic output. '
+                f"{input_handle.solid.describe_node()} cannot be downstream of more than one dynamic output. "
                 f'It is downstream of both "{output_handle.describe()}" and '
                 f'"{self._dynamic_fan_out_index[output_handle.solid_name].describe()}"'
             )
@@ -895,3 +1051,10 @@ class DependencyStructure:
 
     def is_dynamic_mapped(self, solid_name: str) -> bool:
         return solid_name in self._dynamic_fan_out_index
+
+    def has_dynamic_downstreams(self, solid_name: str) -> bool:
+        for upstream_handle in self._dynamic_fan_out_index.values():
+            if upstream_handle.solid_name == solid_name:
+                return True
+
+        return False
