@@ -1,11 +1,12 @@
 import sys
 from contextlib import contextmanager
-from typing import Any, Dict, FrozenSet, Iterator, List, Optional, Tuple, Union
+from typing import Any, Dict, FrozenSet, Iterator, List, Mapping, Optional, Tuple, Union
 
-from dagster import check
+import dagster._check as check
 from dagster.core.definitions import IPipeline, JobDefinition, PipelineDefinition
-from dagster.core.definitions.pipeline import PipelineSubsetDefinition
 from dagster.core.definitions.pipeline_base import InMemoryPipeline
+from dagster.core.definitions.pipeline_definition import PipelineSubsetDefinition
+from dagster.core.definitions.reconstruct import ReconstructablePipeline
 from dagster.core.errors import DagsterExecutionInterruptedError, DagsterInvariantViolationError
 from dagster.core.events import DagsterEvent, EngineEventData
 from dagster.core.execution.context.system import PlanOrchestrationContext
@@ -14,9 +15,9 @@ from dagster.core.execution.plan.outputs import StepOutputHandle
 from dagster.core.execution.plan.plan import ExecutionPlan
 from dagster.core.execution.plan.state import KnownExecutionState
 from dagster.core.execution.retries import RetryMode
-from dagster.core.instance import DagsterInstance
+from dagster.core.instance import DagsterInstance, InstanceRef
 from dagster.core.selector import parse_step_selection
-from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus
+from dagster.core.storage.pipeline_run import DagsterRun, PipelineRun, PipelineRunStatus
 from dagster.core.system_config.objects import ResolvedRunConfig
 from dagster.core.telemetry import log_repo_stats, telemetry_wrapper
 from dagster.core.utils import str_format_set
@@ -74,25 +75,51 @@ def execute_run_iterator(
         return gen_execute_on_cancel()
 
     if not resume_from_failure:
-        # this should be the first run worker to start
-        check.invariant(
-            pipeline_run.status == PipelineRunStatus.NOT_STARTED
-            or pipeline_run.status == PipelineRunStatus.STARTING,
-            desc="Pipeline run {} ({}) in state {}, expected NOT_STARTED or STARTING".format(
-                pipeline_run.pipeline_name, pipeline_run.run_id, pipeline_run.status
-            ),
-        )
+        if pipeline_run.status not in (PipelineRunStatus.NOT_STARTED, PipelineRunStatus.STARTING):
+            if instance.run_monitoring_enabled:
+                # This can happen if the pod was unexpectedly restarted by the cluster - ignore it since
+                # the run monitoring daemon will also spin up a new pod
+                def gen_ignore_duplicate_run_worker():
+                    yield instance.report_engine_event(
+                        "Ignoring a duplicate run that was started from somewhere other than the run monitor daemon",
+                        pipeline_run,
+                    )
+
+                return gen_ignore_duplicate_run_worker()
+            elif pipeline_run.is_finished:
+
+                def gen_ignore_duplicate_run_worker():
+                    yield instance.report_engine_event(
+                        "Ignoring a run worker that started after the run had already finished.",
+                        pipeline_run,
+                    )
+
+                return gen_ignore_duplicate_run_worker()
+            else:
+
+                def gen_fail_restarted_run_worker():
+                    yield instance.report_engine_event(
+                        f"{pipeline_run.pipeline_name} ({pipeline_run.run_id}) started "
+                        f"a new run worker while the run was already in state {pipeline_run.status}. "
+                        "This most frequently happens when the run worker unexpectedly stops and is "
+                        "restarted by the cluster. Marking the run as failed.",
+                        pipeline_run,
+                    )
+                    yield instance.report_run_failed(pipeline_run)
+
+                return gen_fail_restarted_run_worker()
+
     else:
         check.invariant(
             pipeline_run.status == PipelineRunStatus.STARTED
             or pipeline_run.status == PipelineRunStatus.STARTING,
-            desc="Pipeline run {} ({}) in state {}, expected STARTED or STARTING because it's "
+            desc="Run of {} ({}) in state {}, expected STARTED or STARTING because it's "
             "resuming from a run worker failure".format(
                 pipeline_run.pipeline_name, pipeline_run.run_id, pipeline_run.status
             ),
         )
 
-    if pipeline_run.solids_to_execute:
+    if pipeline_run.solids_to_execute or pipeline_run.asset_selection:
         pipeline_def = pipeline.get_definition()
         if isinstance(pipeline_def, PipelineSubsetDefinition):
             check.invariant(
@@ -108,7 +135,10 @@ def execute_run_iterator(
             # note that when we receive the solids to execute via PipelineRun, it won't support
             # solid selection query syntax
             pipeline = pipeline.subset_for_execution_from_existing_pipeline(
-                pipeline_run.solids_to_execute
+                frozenset(pipeline_run.solids_to_execute)
+                if pipeline_run.solids_to_execute
+                else None,
+                asset_selection=pipeline_run.asset_selection,
             )
 
     execution_plan = _get_execution_plan_from_run(pipeline, pipeline_run, instance)
@@ -181,12 +211,12 @@ def execute_run(
     check.invariant(
         pipeline_run.status == PipelineRunStatus.NOT_STARTED
         or pipeline_run.status == PipelineRunStatus.STARTING,
-        desc="Pipeline run {} ({}) in state {}, expected NOT_STARTED or STARTING".format(
+        desc="Run {} ({}) in state {}, expected NOT_STARTED or STARTING".format(
             pipeline_run.pipeline_name, pipeline_run.run_id, pipeline_run.status
         ),
     )
     pipeline_def = pipeline.get_definition()
-    if pipeline_run.solids_to_execute:
+    if pipeline_run.solids_to_execute or pipeline_run.asset_selection:
         if isinstance(pipeline_def, PipelineSubsetDefinition):
             check.invariant(
                 pipeline_run.solids_to_execute == pipeline.solids_to_execute,
@@ -201,11 +231,13 @@ def execute_run(
             # note that when we receive the solids to execute via PipelineRun, it won't support
             # solid selection query syntax
             pipeline = pipeline.subset_for_execution_from_existing_pipeline(
-                pipeline_run.solids_to_execute
+                frozenset(pipeline_run.solids_to_execute)
+                if pipeline_run.solids_to_execute
+                else None,
+                pipeline_run.asset_selection,
             )
 
     execution_plan = _get_execution_plan_from_run(pipeline, pipeline_run, instance)
-
     output_capture: Optional[Dict[StepOutputHandle, Any]] = {}
 
     _execute_run_iterable = ExecuteRunWithPlanIterable(
@@ -419,6 +451,9 @@ def _logged_execute_pipeline(
         solid_selection=solid_selection,
         solids_to_execute=solids_to_execute,
         tags=tags,
+        pipeline_code_origin=(
+            pipeline.get_python_origin() if isinstance(pipeline, ReconstructablePipeline) else None
+        ),
     )
 
     return execute_run(
@@ -437,7 +472,7 @@ def reexecute_pipeline(
     mode: Optional[str] = None,
     preset: Optional[str] = None,
     tags: Optional[Dict[str, Any]] = None,
-    instance: DagsterInstance = None,
+    instance: Optional[DagsterInstance] = None,
     raise_on_error: bool = True,
 ) -> PipelineExecutionResult:
     """Reexecute an existing pipeline run.
@@ -511,6 +546,11 @@ def reexecute_pipeline(
                 step_selection,
             )
 
+        if parent_pipeline_run.asset_selection:
+            pipeline = pipeline.subset_for_execution(
+                solid_selection=None, asset_selection=parent_pipeline_run.asset_selection
+            )
+
         pipeline_run = execute_instance.create_run_for_pipeline(
             pipeline_def=pipeline.get_definition(),
             execution_plan=execution_plan,
@@ -518,6 +558,7 @@ def reexecute_pipeline(
             mode=mode,
             tags=tags,
             solid_selection=parent_pipeline_run.solid_selection,
+            asset_selection=parent_pipeline_run.asset_selection,
             solids_to_execute=parent_pipeline_run.solids_to_execute,
             root_run_id=parent_pipeline_run.root_run_id or parent_pipeline_run.run_id,
             parent_run_id=parent_pipeline_run.run_id,
@@ -539,7 +580,7 @@ def reexecute_pipeline_iterator(
     mode: Optional[str] = None,
     preset: Optional[str] = None,
     tags: Optional[Dict[str, Any]] = None,
-    instance: DagsterInstance = None,
+    instance: Optional[DagsterInstance] = None,
 ) -> Iterator[DagsterEvent]:
     """Reexecute a pipeline iteratively.
 
@@ -633,14 +674,14 @@ def execute_plan_iterator(
     pipeline_run: PipelineRun,
     instance: DagsterInstance,
     retry_mode: Optional[RetryMode] = None,
-    run_config: Optional[dict] = None,
+    run_config: Optional[Mapping[str, object]] = None,
 ) -> Iterator[DagsterEvent]:
     check.inst_param(execution_plan, "execution_plan", ExecutionPlan)
     check.inst_param(pipeline, "pipeline", IPipeline)
     check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
     check.inst_param(instance, "instance", DagsterInstance)
     retry_mode = check.opt_inst_param(retry_mode, "retry_mode", RetryMode, RetryMode.DISABLED)
-    run_config = check.opt_dict_param(run_config, "run_config")
+    run_config = check.opt_mapping_param(run_config, "run_config")
 
     return iter(
         ExecuteRunWithPlanIterable(
@@ -718,26 +759,33 @@ def _get_execution_plan_from_run(
         run_config=pipeline_run.run_config,
         mode=pipeline_run.mode,
         step_keys_to_execute=pipeline_run.step_keys_to_execute,
+        instance_ref=instance.get_ref() if instance.is_persistent else None,
     )
 
 
 def create_execution_plan(
     pipeline: Union[IPipeline, PipelineDefinition],
-    run_config: Optional[dict] = None,
+    run_config: Optional[Mapping[str, object]] = None,
     mode: Optional[str] = None,
     step_keys_to_execute: Optional[List[str]] = None,
-    known_state: KnownExecutionState = None,
-    instance: Optional[DagsterInstance] = None,
+    known_state: Optional[KnownExecutionState] = None,
+    instance_ref: Optional[InstanceRef] = None,
     tags: Optional[Dict[str, str]] = None,
 ) -> ExecutionPlan:
     pipeline = _check_pipeline(pipeline)
     pipeline_def = pipeline.get_definition()
     check.inst_param(pipeline_def, "pipeline_def", PipelineDefinition)
-    run_config = check.opt_dict_param(run_config, "run_config", key_type=str)
+    run_config = check.opt_mapping_param(run_config, "run_config", key_type=str)
     mode = check.opt_str_param(mode, "mode", default=pipeline_def.get_default_mode_name())
     check.opt_nullable_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
-    check.opt_inst_param(instance, "instance", DagsterInstance)
+    check.opt_inst_param(instance_ref, "instance_ref", InstanceRef)
     tags = check.opt_dict_param(tags, "tags", key_type=str, value_type=str)
+    known_state = check.opt_inst_param(
+        known_state,
+        "known_state",
+        KnownExecutionState,
+        default=KnownExecutionState(),
+    )
 
     resolved_run_config = ResolvedRunConfig.build(pipeline_def, run_config, mode=mode)
 
@@ -746,7 +794,7 @@ def create_execution_plan(
         resolved_run_config,
         step_keys_to_execute=step_keys_to_execute,
         known_state=known_state,
-        instance=instance,
+        instance_ref=instance_ref,
         tags=tags,
     )
 
@@ -787,7 +835,7 @@ def pipeline_execution_iterator(
         pipeline_canceled_info = serializable_error_info_from_exc_info(sys.exc_info())
         if pipeline_context.raise_on_error:
             raise
-    except Exception:  # pylint: disable=broad-except
+    except BaseException:
         pipeline_exception_info = serializable_error_info_from_exc_info(sys.exc_info())
         if pipeline_context.raise_on_error:
             raise  # finally block will run before this is re-raised
@@ -796,21 +844,28 @@ def pipeline_execution_iterator(
             reloaded_run = pipeline_context.instance.get_run_by_id(pipeline_context.run_id)
             if reloaded_run and reloaded_run.status == PipelineRunStatus.CANCELING:
                 event = DagsterEvent.pipeline_canceled(pipeline_context, pipeline_canceled_info)
+            elif reloaded_run and reloaded_run.status == PipelineRunStatus.CANCELED:
+                # This happens if the run was force-terminated but was still able to send
+                # a cancellation request
+                event = DagsterEvent.engine_event(
+                    pipeline_context,
+                    "Computational resources were cleaned up after the run was forcibly marked as canceled.",
+                    EngineEventData(),
+                )
+            elif pipeline_context.instance.run_will_resume(pipeline_context.run_id):
+                event = DagsterEvent.engine_event(
+                    pipeline_context,
+                    "Execution was interrupted unexpectedly. "
+                    "No user initiated termination request was found, not treating as failure because run will be resumed.",
+                    EngineEventData(),
+                )
             else:
-                if pipeline_context.instance.run_will_resume(pipeline_context.run_id):
-                    event = DagsterEvent.engine_event(
-                        pipeline_context,
-                        "Execution was interrupted unexpectedly. "
-                        "No user initiated termination request was found, not treating as failure because run will be resumed.",
-                        EngineEventData(),
-                    )
-                else:
-                    event = DagsterEvent.pipeline_failure(
-                        pipeline_context,
-                        "Execution was interrupted unexpectedly. "
-                        "No user initiated termination request was found, treating as failure.",
-                        pipeline_canceled_info,
-                    )
+                event = DagsterEvent.pipeline_failure(
+                    pipeline_context,
+                    "Execution was interrupted unexpectedly. "
+                    "No user initiated termination request was found, treating as failure.",
+                    pipeline_canceled_info,
+                )
         elif pipeline_exception_info:
             event = DagsterEvent.pipeline_failure(
                 pipeline_context,
@@ -885,7 +940,7 @@ def _check_execute_pipeline_args(
     Optional[dict],
     Optional[str],
     Dict[str, Any],
-    FrozenSet[str],
+    Optional[FrozenSet[str]],
     Optional[List[str]],
 ]:
     pipeline = _check_pipeline(pipeline)
@@ -984,18 +1039,19 @@ def _resolve_reexecute_step_selection(
     pipeline: IPipeline,
     mode: Optional[str],
     run_config: Optional[dict],
-    parent_pipeline_run: PipelineRun,
+    parent_pipeline_run: DagsterRun,
     step_selection: List[str],
 ) -> ExecutionPlan:
     if parent_pipeline_run.solid_selection:
-        pipeline = pipeline.subset_for_execution(parent_pipeline_run.solid_selection)
+        pipeline = pipeline.subset_for_execution(parent_pipeline_run.solid_selection, None)
+
+    state = KnownExecutionState.build_for_reexecution(instance, parent_pipeline_run)
 
-    parent_logs = instance.all_logs(parent_pipeline_run.run_id)
     parent_plan = create_execution_plan(
         pipeline,
         parent_pipeline_run.run_config,
         mode,
-        known_state=KnownExecutionState.derive_from_logs(parent_logs),
+        known_state=state,
     )
     step_keys_to_execute = parse_step_selection(parent_plan.get_all_step_deps(), step_selection)
     execution_plan = create_execution_plan(
@@ -1003,7 +1059,7 @@ def _resolve_reexecute_step_selection(
         run_config,
         mode,
         step_keys_to_execute=list(step_keys_to_execute),
-        known_state=KnownExecutionState.for_reexecution(parent_logs, step_keys_to_execute),
+        known_state=state.update_for_step_selection(step_keys_to_execute),
         tags=parent_pipeline_run.tags,
     )
     return execution_plan
