@@ -1,22 +1,33 @@
 import warnings
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union, cast
+from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Optional, Sequence, Union, cast
 
-from dagster import check
-from dagster.core.definitions.events import AssetKey
-from dagster.core.definitions.op_def import OpDefinition
-from dagster.core.definitions.solid import SolidDefinition
+import dagster._check as check
+from dagster.core.definitions.asset_layer import AssetOutputInfo
+from dagster.core.definitions.events import (
+    AssetKey,
+    AssetMaterialization,
+    AssetObservation,
+    Materialization,
+    MetadataEntry,
+    PartitionMetadataEntry,
+)
+from dagster.core.definitions.op_definition import OpDefinition
+from dagster.core.definitions.partition_key_range import PartitionKeyRange
+from dagster.core.definitions.solid_definition import SolidDefinition
+from dagster.core.definitions.time_window_partitions import TimeWindow
 from dagster.core.errors import DagsterInvariantViolationError
 from dagster.core.execution.plan.utils import build_resources_for_manager
 
 if TYPE_CHECKING:
-    from dagster.core.execution.context.system import StepExecutionContext
-    from dagster.core.types.dagster_type import DagsterType
     from dagster.core.definitions import PipelineDefinition
+    from dagster.core.definitions.resource_definition import Resources
+    from dagster.core.events import DagsterEvent
+    from dagster.core.execution.context.system import StepExecutionContext
+    from dagster.core.execution.plan.outputs import StepOutputHandle
+    from dagster.core.execution.plan.plan import ExecutionPlan
     from dagster.core.log_manager import DagsterLogManager
     from dagster.core.system_config.objects import ResolvedRunConfig
-    from dagster.core.definitions.resource import Resources
-    from dagster.core.execution.plan.plan import ExecutionPlan
-    from dagster.core.execution.plan.outputs import StepOutputHandle
+    from dagster.core.types.dagster_type import DagsterType
 
 RUN_ID_PLACEHOLDER = "__EPHEMERAL_RUN_ID"
 
@@ -43,6 +54,8 @@ class OutputContext:
         resources (Optional[Resources]): The resources required by the output manager, specified by the
             `required_resource_keys` parameter.
         op_def (Optional[OpDefinition]): The definition of the op that produced the output.
+        asset_info: Optional[AssetOutputInfo]: (Experimental) Asset info corresponding to the
+            output.
     """
 
     def __init__(
@@ -62,8 +75,10 @@ class OutputContext:
         resources: Optional[Union["Resources", Dict[str, Any]]] = None,
         step_context: Optional["StepExecutionContext"] = None,
         op_def: Optional["OpDefinition"] = None,
+        asset_info: Optional[AssetOutputInfo] = None,
+        warn_on_step_context_use: bool = False,
     ):
-        from dagster.core.definitions.resource import Resources, IContainsGenerator
+        from dagster.core.definitions.resource_definition import IContainsGenerator, Resources
         from dagster.core.execution.build_resources import build_resources
 
         self._step_key = step_key
@@ -82,6 +97,8 @@ class OutputContext:
         self._version = version
         self._resource_config = resource_config
         self._step_context = step_context
+        self._asset_info = asset_info
+        self._warn_on_step_context_use = warn_on_step_context_use
 
         if isinstance(resources, Resources):
             self._resources_cm = None
@@ -94,6 +111,10 @@ class OutputContext:
             self._resources_contain_cm = isinstance(self._resources, IContainsGenerator)
             self._cm_scope_entered = False
 
+        self._events: List["DagsterEvent"] = []
+        self._user_events: List[Union[AssetMaterialization, AssetObservation, Materialization]] = []
+        self._metadata_entries: Optional[List[Union[MetadataEntry, PartitionMetadataEntry]]] = None
+
     def __enter__(self):
         if self._resources_cm:
             self._cm_scope_entered = True
@@ -224,17 +245,33 @@ class OutputContext:
         return self._resources
 
     @property
-    def asset_key(self) -> Optional[AssetKey]:
-        matching_output_defs = [
-            output_def
-            for output_def in cast(SolidDefinition, self._solid_def).output_defs
-            if output_def.name == self.name
-        ]
-        check.invariant(len(matching_output_defs) == 1)
-        return matching_output_defs[0].get_asset_key(self)
+    def asset_info(self) -> Optional[AssetOutputInfo]:
+        return self._asset_info
+
+    @property
+    def has_asset_key(self) -> bool:
+        return self._asset_info is not None
+
+    @property
+    def asset_key(self) -> AssetKey:
+        if self._asset_info is None:
+            raise DagsterInvariantViolationError(
+                "Attempting to access asset_key, "
+                "but it was not provided when constructing the OutputContext"
+            )
+
+        return self._asset_info.key
 
     @property
     def step_context(self) -> "StepExecutionContext":
+        if self._warn_on_step_context_use:
+            warnings.warn(
+                "You are using InputContext.upstream_output.step_context"
+                "This use on upstream_output is deprecated and will fail in the future"
+                "Try to obtain what you need directly from InputContext"
+                "For more details: https://github.com/dagster-io/dagster/issues/7900"
+            )
+
         if self._step_context is None:
             raise DagsterInvariantViolationError(
                 "Attempting to access step_context, "
@@ -243,6 +280,101 @@ class OutputContext:
 
         return self._step_context
 
+    @property
+    def has_partition_key(self) -> bool:
+        """Whether the current run is a partitioned run"""
+        if self._warn_on_step_context_use:
+            warnings.warn(
+                "You are using InputContext.upstream_output.has_partition_key"
+                "This use on upstream_output is deprecated and will fail in the future"
+                "Try to obtain what you need directly from InputContext"
+                "For more details: https://github.com/dagster-io/dagster/issues/7900"
+            )
+
+        return self.step_context.has_partition_key
+
+    @property
+    def partition_key(self) -> str:
+        """The partition key for the current run.
+
+        Raises an error if the current run is not a partitioned run.
+        """
+        if self._warn_on_step_context_use:
+            warnings.warn(
+                "You are using InputContext.upstream_output.partition_key"
+                "This use on upstream_output is deprecated and will fail in the future"
+                "Try to obtain what you need directly from InputContext"
+                "For more details: https://github.com/dagster-io/dagster/issues/7900"
+            )
+
+        return self.step_context.partition_key
+
+    @property
+    def has_asset_partitions(self) -> bool:
+        if self._warn_on_step_context_use:
+            warnings.warn(
+                "You are using InputContext.upstream_output.has_asset_partitions"
+                "This use on upstream_output is deprecated and will fail in the future"
+                "Try to obtain what you need directly from InputContext"
+                "For more details: https://github.com/dagster-io/dagster/issues/7900"
+            )
+
+        if self._step_context is not None:
+            return self._step_context.has_asset_partitions_for_output(self.name)
+        else:
+            return False
+
+    @property
+    def asset_partition_key(self) -> str:
+        """The partition key for output asset.
+
+        Raises an error if the output asset has no partitioning, or if the run covers a partition
+        range for the output asset.
+        """
+        if self._warn_on_step_context_use:
+            warnings.warn(
+                "You are using InputContext.upstream_output.asset_partition_key"
+                "This use on upstream_output is deprecated and will fail in the future"
+                "Try to obtain what you need directly from InputContext"
+                "For more details: https://github.com/dagster-io/dagster/issues/7900"
+            )
+
+        return self.step_context.asset_partition_key_for_output(self.name)
+
+    @property
+    def asset_partition_key_range(self) -> PartitionKeyRange:
+        """The partition key range for output asset.
+
+        Raises an error if the output asset has no partitioning.
+        """
+        if self._warn_on_step_context_use:
+            warnings.warn(
+                "You are using InputContext.upstream_output.asset_partition_key_range"
+                "This use on upstream_output is deprecated and will fail in the future"
+                "Try to obtain what you need directly from InputContext"
+                "For more details: https://github.com/dagster-io/dagster/issues/7900"
+            )
+
+        return self.step_context.asset_partition_key_range_for_output(self.name)
+
+    @property
+    def asset_partitions_time_window(self) -> TimeWindow:
+        """The time window for the partitions of the output asset.
+
+        Raises an error if either of the following are true:
+        - The output asset has no partitioning.
+        - The output asset is not partitioned with a TimeWindowPartitionsDefinition.
+        """
+        if self._warn_on_step_context_use:
+            warnings.warn(
+                "You are using InputContext.upstream_output.asset_partitions_time_window"
+                "This use on upstream_output is deprecated and will fail in the future"
+                "Try to obtain what you need directly from InputContext"
+                "For more details: https://github.com/dagster-io/dagster/issues/7900"
+            )
+
+        return self.step_context.asset_partitions_time_window_for_output(self.name)
+
     def get_run_scoped_output_identifier(self) -> List[str]:
         """Utility method to get a collection of identifiers that as a whole represent a unique
         step output.
@@ -262,7 +394,7 @@ class OutputContext:
 
         warnings.warn(
             "`OutputContext.get_run_scoped_output_identifier` is deprecated. Use "
-            "`OutputContext.get_output_identifier` instead."
+            "`OutputContext.get_identifier` instead."
         )
         # if run_id is None and this is a re-execution, it means we failed to find its source run id
         check.invariant(
@@ -286,7 +418,7 @@ class OutputContext:
 
         return [run_id, step_key, name]
 
-    def get_output_identifier(self) -> List[str]:
+    def get_identifier(self) -> List[str]:
         """Utility method to get a collection of identifiers that as a whole represent a unique
         step output.
 
@@ -323,6 +455,148 @@ class OutputContext:
 
         return identifier
 
+    def get_output_identifier(self) -> List[str]:
+        warnings.warn(
+            "`OutputContext.get_output_identifier` is deprecated. Use "
+            "`OutputContext.get_identifier` instead."
+        )
+
+        return self.get_identifier()
+
+    def get_asset_identifier(self) -> Sequence[str]:
+        if self.asset_key is not None:
+            if self.has_asset_partitions:
+                return self.asset_key.path + [self.asset_partition_key]
+            else:
+                return self.asset_key.path
+        else:
+            check.failed("Can't get asset output identifier for an output with no asset key")
+
+    def get_asset_output_identifier(self) -> Sequence[str]:
+        warnings.warn(
+            "`OutputContext.get_asset_output_identifier` is deprecated. Use "
+            "`OutputContext.get_asset_identifier` instead."
+        )
+
+        return self.get_asset_identifier()
+
+    def log_event(
+        self, event: Union[AssetObservation, AssetMaterialization, Materialization]
+    ) -> None:
+        """Log an AssetMaterialization or AssetObservation from within the body of an io manager's `handle_output` method.
+
+        Events logged with this method will appear in the event log.
+
+        Args:
+            event (Union[AssetMaterialization, Materialization, AssetObservation]): The event to log.
+
+        Examples:
+
+        .. code-block:: python
+
+            from dagster import IOManager, AssetMaterialization
+
+            class MyIOManager(IOManager):
+                def handle_output(self, context, obj):
+                    context.log_event(AssetMaterialization("foo"))
+        """
+        from dagster.core.events import DagsterEvent
+
+        if isinstance(event, (AssetMaterialization, Materialization)):
+            if self._step_context:
+                self._events.append(
+                    DagsterEvent.asset_materialization(
+                        self._step_context,
+                        event,
+                        self._step_context.get_input_lineage(),
+                    )
+                )
+            self._user_events.append(event)
+        elif isinstance(event, AssetObservation):
+            if self._step_context:
+                self._events.append(DagsterEvent.asset_observation(self._step_context, event))
+            self._user_events.append(event)
+        else:
+            check.failed("Unexpected event {event}".format(event=event))
+
+    def consume_events(self) -> Iterator["DagsterEvent"]:
+        """Pops and yields all user-generated events that have been recorded from this context.
+
+        If consume_events has not yet been called, this will yield all logged events since the call to `handle_output`. If consume_events has been called, it will yield all events since the last time consume_events was called. Designed for internal use. Users should never need to invoke this method.
+        """
+
+        events = self._events
+        self._events = []
+        yield from events
+
+    def get_logged_events(
+        self,
+    ) -> List[Union[AssetMaterialization, Materialization, AssetObservation]]:
+        """Retrieve the list of user-generated events that were logged via the context.
+
+
+        User-generated events that were yielded will not appear in this list.
+
+        **Examples:**
+
+        .. code-block:: python
+
+            from dagster import IOManager, build_output_context, AssetMaterialization
+
+            class MyIOManager(IOManager):
+                def handle_output(self, context, obj):
+                    ...
+
+            def test_handle_output():
+                mgr = MyIOManager()
+                context = build_output_context()
+                mgr.handle_output(context)
+                all_user_events = context.get_logged_events()
+                materializations = [event for event in all_user_events if isinstance(event, AssetMaterialization)]
+                ...
+        """
+
+        return self._user_events
+
+    def add_output_metadata(self, metadata: Dict[str, Any]) -> None:
+        """Add a dictionary of metadata to the handled output.
+
+        Metadata entries added will show up in the HANDLED_OUTPUT and ASSET_MATERIALIZATION events for the run.
+
+        Args:
+            metadata (Dict[str, Any]): A metadata dictionary to log
+
+        Examples:
+
+        .. code-block:: python
+
+            from dagster import IOManager
+
+            class MyIOManager(IOManager):
+                def handle_output(self, context, obj):
+                    context.add_output_metadata({"foo": "bar"})
+        """
+        from dagster.core.definitions.metadata import normalize_metadata
+
+        self._metadata_entries = normalize_metadata(metadata, [])
+
+    def get_logged_metadata_entries(
+        self,
+    ) -> List[Union[MetadataEntry, PartitionMetadataEntry]]:
+        """Get the list of metadata entries that have been logged for use with this output."""
+        return self._metadata_entries or []
+
+    def consume_logged_metadata_entries(
+        self,
+    ) -> List[Union[MetadataEntry, PartitionMetadataEntry]]:
+        """Pops and yields all user-generated metadata entries that have been recorded from this context.
+
+        If consume_logged_metadata_entries has not yet been called, this will yield all logged events since the call to `handle_output`. If consume_logged_metadata_entries has been called, it will yield all events since the last time consume_logged_metadata_entries was called. Designed for internal use. Users should never need to invoke this method.
+        """
+        result = self._metadata_entries
+        self._metadata_entries = []
+        return result or []
+
 
 def get_output_context(
     execution_plan: "ExecutionPlan",
@@ -334,6 +608,7 @@ def get_output_context(
     step_context: Optional["StepExecutionContext"],
     resources: Optional["Resources"],
     version: Optional[str],
+    warn_on_step_context_use: bool = False,
 ) -> "OutputContext":
     """
     Args:
@@ -357,6 +632,11 @@ def get_output_context(
     io_manager_key = output_def.io_manager_key
     resource_config = resolved_run_config.resources[io_manager_key].config
 
+    node_handle = execution_plan.get_step_by_key(step.key).solid_handle
+    asset_info = pipeline_def.asset_layer.asset_info_for_output(
+        node_handle=node_handle, output_name=step_output.name
+    )
+
     if step_context:
         check.invariant(
             not resources,
@@ -381,6 +661,8 @@ def get_output_context(
         step_context=step_context,
         resource_config=resource_config,
         resources=resources,
+        asset_info=asset_info,
+        warn_on_step_context_use=warn_on_step_context_use,
     )
 
 
@@ -415,6 +697,7 @@ def build_output_context(
     resources: Optional[Dict[str, Any]] = None,
     solid_def: Optional[SolidDefinition] = None,
     op_def: Optional[OpDefinition] = None,
+    asset_key: Optional[Union[AssetKey, str]] = None,
 ) -> "OutputContext":
     """Builds output context from provided parameters.
 
@@ -438,7 +721,9 @@ def build_output_context(
             For a given key, you can provide either an actual instance of an object, or a resource
             definition.
         solid_def (Optional[SolidDefinition]): The definition of the solid that produced the output.
-        op_def (Optional[OpDefinition]): The definition of the solid that produced the output.
+        op_def (Optional[OpDefinition]): The definition of the op that produced the output.
+        asset_key: Optional[Union[AssetKey, Sequence[str], str]]: The asset key corresponding to the
+            output.
 
     Examples:
 
@@ -450,8 +735,8 @@ def build_output_context(
                 do_something
 
     """
-    from dagster.core.types.dagster_type import DagsterType
     from dagster.core.execution.context_creation_pipeline import initialize_console_manager
+    from dagster.core.types.dagster_type import DagsterType
 
     step_key = check.opt_str_param(step_key, "step_key")
     name = check.opt_str_param(name, "name")
@@ -464,6 +749,7 @@ def build_output_context(
     resources = check.opt_dict_param(resources, "resources", key_type=str)
     solid_def = check.opt_inst_param(solid_def, "solid_def", SolidDefinition)
     op_def = check.opt_inst_param(op_def, "op_def", OpDefinition)
+    asset_key = AssetKey.from_coerceable(asset_key) if asset_key else None
 
     return OutputContext(
         step_key=step_key,
@@ -481,4 +767,5 @@ def build_output_context(
         resources=resources,
         step_context=None,
         op_def=op_def,
+        asset_info=AssetOutputInfo(key=asset_key) if asset_key else None,
     )
