@@ -1,16 +1,20 @@
 import warnings
 from collections import OrderedDict
-from typing import Optional, Sequence
+from typing import TYPE_CHECKING, List, Optional, Sequence, Union
 
-from dagster import check
+import dagster._check as check
 from dagster.core.definitions.events import AssetKey
-from dagster.core.definitions.run_request import JobType
-from dagster.core.definitions.sensor import DEFAULT_SENSOR_DAEMON_INTERVAL
-from dagster.core.errors import DagsterInvariantViolationError
+from dagster.core.definitions.run_request import InstigatorType
+from dagster.core.definitions.schedule_definition import DefaultScheduleStatus
+from dagster.core.definitions.sensor_definition import (
+    DEFAULT_SENSOR_DAEMON_INTERVAL,
+    DefaultSensorStatus,
+)
 from dagster.core.execution.plan.handle import ResolvedFromDynamicStepHandle, StepHandle
 from dagster.core.origin import PipelinePythonOrigin
 from dagster.core.snap import ExecutionPlanSnapshot
 from dagster.core.utils import toposort
+from dagster.serdes import create_snapshot_id
 from dagster.utils.schedules import schedule_execution_time_iterator
 
 from .external_data import (
@@ -21,9 +25,13 @@ from .external_data import (
     ExternalScheduleData,
     ExternalSensorData,
 )
-from .handle import JobHandle, PartitionSetHandle, PipelineHandle, RepositoryHandle
+from .handle import InstigatorHandle, PartitionSetHandle, PipelineHandle, RepositoryHandle
 from .pipeline_index import PipelineIndex
 from .represented import RepresentedPipeline
+from .selector import InstigatorSelector, RepositorySelector
+
+if TYPE_CHECKING:
+    from dagster.core.scheduler.instigation import InstigatorState
 
 
 class ExternalRepository:
@@ -33,37 +41,31 @@ class ExternalRepository:
     objects such as these to interact with user-defined artifacts.
     """
 
-    def __init__(self, external_repository_data, repository_handle):
+    def __init__(
+        self, external_repository_data: ExternalRepositoryData, repository_handle: RepositoryHandle
+    ):
         self.external_repository_data = check.inst_param(
             external_repository_data, "external_repository_data", ExternalRepositoryData
         )
-        self._pipeline_index_map = OrderedDict(
-            (
-                external_pipeline_data.pipeline_snapshot.name,
-                PipelineIndex(
-                    external_pipeline_data.pipeline_snapshot,
-                    external_pipeline_data.parent_pipeline_snapshot,
-                ),
+        self._pipeline_index_map = OrderedDict()
+        self._job_index_map = OrderedDict()
+        for external_pipeline_data in external_repository_data.external_pipeline_datas:
+            key = external_pipeline_data.pipeline_snapshot.name
+            index = PipelineIndex(
+                external_pipeline_data.pipeline_snapshot,
+                external_pipeline_data.parent_pipeline_snapshot,
             )
-            for external_pipeline_data in external_repository_data.external_pipeline_datas
-        )
-        self._job_index_map = OrderedDict(
-            (
-                external_pipeline_data.pipeline_snapshot.name,
-                PipelineIndex(
-                    external_pipeline_data.pipeline_snapshot,
-                    external_pipeline_data.parent_pipeline_snapshot,
-                ),
-            )
-            for external_pipeline_data in external_repository_data.external_pipeline_datas
-            if external_pipeline_data.is_job
-        )
+            self._pipeline_index_map[key] = index
+            if external_pipeline_data.is_job:
+                self._job_index_map[key] = index
+
         self._handle = check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
 
-        instigation_list = (
-            external_repository_data.external_schedule_datas
-            + external_repository_data.external_sensor_datas
-        )
+        # mypy doesn't understand splat
+        instigation_list: Sequence[Union[ExternalScheduleData, ExternalSensorData]] = [
+            *external_repository_data.external_schedule_datas,  # type: ignore
+            *external_repository_data.external_sensor_datas,  # type: ignore
+        ]
         self._instigation_map = OrderedDict(
             (instigation_data.name, instigation_data) for instigation_data in instigation_list
         )
@@ -72,6 +74,17 @@ class ExternalRepository:
             for external_partition_set_data in external_repository_data.external_partition_set_datas
         )
 
+        # pylint: disable=unsubscriptable-object
+        _asset_jobs: OrderedDict[str, List[ExternalAssetNode]] = OrderedDict()
+        for asset_node in external_repository_data.external_asset_graph_data:
+            for job_name in asset_node.job_names:
+                if job_name not in _asset_jobs:
+                    _asset_jobs[job_name] = [asset_node]
+                else:
+                    _asset_jobs[job_name].append(asset_node)
+        # pylint: disable=unsubscriptable-object
+        self._asset_jobs: OrderedDict[str, Sequence[ExternalAssetNode]] = OrderedDict(_asset_jobs)
+
     @property
     def name(self):
         return self.external_repository_data.name
@@ -131,7 +144,7 @@ class ExternalRepository:
             for external_partition_set_data in self.external_repository_data.external_partition_set_datas
         ]
 
-    def get_full_external_pipeline(self, pipeline_name):
+    def get_full_external_pipeline(self, pipeline_name: str) -> "ExternalPipeline":
         check.str_param(pipeline_name, "pipeline_name")
         return ExternalPipeline(
             self.external_repository_data.get_external_pipeline_data(pipeline_name),
@@ -145,7 +158,7 @@ class ExternalRepository:
     def has_external_job(self, job_name):
         return job_name in self._job_index_map
 
-    def get_external_job(self, job_name):
+    def get_external_job(self, job_name) -> "ExternalPipeline":
         check.str_param(job_name, "job_name")
 
         if not self.has_external_job(job_name):
@@ -157,13 +170,19 @@ class ExternalRepository:
             pipeline_index=self.get_pipeline_index(job_name),
         )
 
-    def get_external_jobs(self):
+    def get_external_jobs(self) -> List["ExternalPipeline"]:
         return [self.get_external_job(pn) for pn in self._job_index_map]
 
     @property
     def handle(self):
         return self._handle
 
+    @property
+    def selector_id(self):
+        return create_snapshot_id(
+            RepositorySelector(self._handle.location_name, self._handle.repository_name)
+        )
+
     def get_external_origin(self):
         return self.handle.get_external_origin()
 
@@ -177,10 +196,14 @@ class ExternalRepository:
         """
         return self.get_external_origin().get_id()
 
-    def get_external_asset_nodes(self) -> Sequence[ExternalAssetNode]:
-        return self.external_repository_data.external_asset_graph_data
+    def get_external_asset_nodes(self, job_name=None) -> Sequence[ExternalAssetNode]:
+        return (
+            self.external_repository_data.external_asset_graph_data
+            if job_name is None
+            else self._asset_jobs.get(job_name, [])
+        )
 
-    def get_external_asset_node(self, asset_key: AssetKey) -> ExternalAssetNode:
+    def get_external_asset_node(self, asset_key: AssetKey) -> Optional[ExternalAssetNode]:
         matching = [
             asset_node
             for asset_node in self.external_repository_data.external_asset_graph_data
@@ -252,6 +275,14 @@ class ExternalPipeline(RepresentedPipeline):
             else None
         )
 
+    @property
+    def asset_selection(self):
+        return (
+            self._pipeline_index.pipeline_snapshot.lineage_snapshot.asset_selection
+            if self._pipeline_index.pipeline_snapshot.lineage_snapshot
+            else None
+        )
+
     @property
     def active_presets(self):
         return list(self._active_preset_dict.values())
@@ -286,13 +317,17 @@ class ExternalPipeline(RepresentedPipeline):
             mode_name if mode_name else self.get_default_mode_name()
         ).root_config_key
 
-    def get_default_mode_name(self):
+    def get_default_mode_name(self) -> str:
         return self._pipeline_index.get_default_mode_name()
 
     @property
     def tags(self):
         return self._pipeline_index.pipeline_snapshot.tags
 
+    @property
+    def metadata(self):
+        return self._pipeline_index.pipeline_snapshot.metadata
+
     @property
     def computed_pipeline_snapshot_id(self):
         return self._pipeline_index.pipeline_snapshot_id
@@ -340,25 +375,13 @@ class ExternalExecutionPlan:
     was compiled in another process or persisted in an instance.
     """
 
-    def __init__(self, execution_plan_snapshot, represented_pipeline):
+    def __init__(self, execution_plan_snapshot):
         self.execution_plan_snapshot = check.inst_param(
             execution_plan_snapshot, "execution_plan_snapshot", ExecutionPlanSnapshot
         )
-        self.represented_pipeline = check.inst_param(
-            represented_pipeline, "represented_pipeline", RepresentedPipeline
-        )
 
         self._step_index = {step.key: step for step in self.execution_plan_snapshot.steps}
 
-        if (
-            execution_plan_snapshot.pipeline_snapshot_id
-            != represented_pipeline.identifying_pipeline_snapshot_id
-        ):
-            raise DagsterInvariantViolationError(
-                "The target snapshot ID from the execution plan snapshot does not match the "
-                "passed in target snapshot. "
-            )
-
         self._step_keys_in_plan = (
             set(execution_plan_snapshot.step_keys_to_execute)
             if execution_plan_snapshot.step_keys_to_execute
@@ -437,7 +460,7 @@ class ExternalSchedule:
         self._external_schedule_data = check.inst_param(
             external_schedule_data, "external_schedule_data", ExternalScheduleData
         )
-        self._handle = JobHandle(
+        self._handle = InstigatorHandle(
             self._external_schedule_data.name, check.inst_param(handle, "handle", RepositoryHandle)
         )
 
@@ -487,21 +510,56 @@ class ExternalSchedule:
     def get_external_origin_id(self):
         return self.get_external_origin().get_id()
 
-    # ScheduleState that represents the state of the schedule
-    # when there is no row in the schedule DB (for example, when
-    # the schedule is first created in code)
-    def get_default_instigation_state(self, instance):
-        from dagster.core.scheduler.job import JobState, JobStatus, ScheduleJobData
-
-        return JobState(
-            self.get_external_origin(),
-            JobType.SCHEDULE,
-            JobStatus.STOPPED,
-            ScheduleJobData(
-                self.cron_schedule, start_timestamp=None, scheduler=instance.scheduler_class
-            ),
+    @property
+    def selector_id(self):
+        return create_snapshot_id(
+            InstigatorSelector(
+                self.handle.location_name,
+                self.handle.repository_name,
+                self._external_schedule_data.name,
+            )
         )
 
+    @property
+    def default_status(self) -> DefaultScheduleStatus:
+        return (
+            self._external_schedule_data.default_status
+            if self._external_schedule_data.default_status
+            else DefaultScheduleStatus.STOPPED
+        )
+
+    def get_current_instigator_state(self, stored_state: Optional["InstigatorState"]):
+        from dagster.core.scheduler.instigation import (
+            InstigatorState,
+            InstigatorStatus,
+            ScheduleInstigatorData,
+        )
+
+        if self.default_status == DefaultScheduleStatus.RUNNING:
+            if stored_state:
+                return stored_state
+
+            return InstigatorState(
+                self.get_external_origin(),
+                InstigatorType.SCHEDULE,
+                InstigatorStatus.AUTOMATICALLY_RUNNING,
+                ScheduleInstigatorData(self.cron_schedule, start_timestamp=None),
+            )
+        else:
+            # Ignore AUTOMATICALLY_RUNNING states in the DB if the default status
+            # isn't DefaultScheduleStatus.RUNNING - this would indicate that the schedule's
+            # default has been changed in code but there's still a lingering AUTOMATICALLY_RUNNING
+            # row in the database that can be ignored
+            if stored_state and stored_state.status != InstigatorStatus.AUTOMATICALLY_RUNNING:
+                return stored_state
+
+            return InstigatorState(
+                self.get_external_origin(),
+                InstigatorType.SCHEDULE,
+                InstigatorStatus.STOPPED,
+                ScheduleInstigatorData(self.cron_schedule, start_timestamp=None),
+            )
+
     def execution_time_iterator(self, start_timestamp):
         return schedule_execution_time_iterator(
             start_timestamp, self.cron_schedule, self.execution_timezone
@@ -513,7 +571,7 @@ class ExternalSensor:
         self._external_sensor_data = check.inst_param(
             external_sensor_data, "external_sensor_data", ExternalSensorData
         )
-        self._handle = JobHandle(
+        self._handle = InstigatorHandle(
             self._external_sensor_data.name, check.inst_param(handle, "handle", RepositoryHandle)
         )
 
@@ -521,6 +579,10 @@ class ExternalSensor:
     def name(self):
         return self._external_sensor_data.name
 
+    @property
+    def handle(self):
+        return self._handle
+
     @property
     def pipeline_name(self):
         target = self._get_single_target()
@@ -570,14 +632,58 @@ class ExternalSensor:
     def get_external_origin_id(self):
         return self.get_external_origin().get_id()
 
-    def get_default_instigation_state(self, _instance):
-        from dagster.core.scheduler.job import JobState, JobStatus, SensorJobData
+    @property
+    def selector_id(self):
+        return create_snapshot_id(
+            InstigatorSelector(
+                self.handle.location_name,
+                self.handle.repository_name,
+                self._external_sensor_data.name,
+            )
+        )
 
-        return JobState(
-            self.get_external_origin(),
-            JobType.SENSOR,
-            JobStatus.STOPPED,
-            SensorJobData(min_interval=self.min_interval_seconds),
+    def get_current_instigator_state(self, stored_state: Optional["InstigatorState"]):
+        from dagster.core.scheduler.instigation import (
+            InstigatorState,
+            InstigatorStatus,
+            SensorInstigatorData,
+        )
+
+        if self.default_status == DefaultSensorStatus.RUNNING:
+            return (
+                stored_state
+                if stored_state
+                else InstigatorState(
+                    self.get_external_origin(),
+                    InstigatorType.SENSOR,
+                    InstigatorStatus.AUTOMATICALLY_RUNNING,
+                    SensorInstigatorData(min_interval=self.min_interval_seconds),
+                )
+            )
+        else:
+            # Ignore AUTOMATICALLY_RUNNING states in the DB if the default status
+            # isn't DefaultSensorStatus.RUNNING - this would indicate that the schedule's
+            # default has changed
+            if stored_state and stored_state.status != InstigatorStatus.AUTOMATICALLY_RUNNING:
+                return stored_state
+
+            return InstigatorState(
+                self.get_external_origin(),
+                InstigatorType.SENSOR,
+                InstigatorStatus.STOPPED,
+                SensorInstigatorData(min_interval=self.min_interval_seconds),
+            )
+
+    @property
+    def metadata(self):
+        return self._external_sensor_data.metadata
+
+    @property
+    def default_status(self) -> DefaultSensorStatus:
+        return (
+            self._external_sensor_data.default_status
+            if self._external_sensor_data
+            else DefaultSensorStatus.STOPPED
         )
 
 
@@ -606,6 +712,10 @@ class ExternalPartitionSet:
     def pipeline_name(self):
         return self._external_partition_set_data.pipeline_name
 
+    @property
+    def repository_handle(self):
+        return self._handle.repository_handle
+
     def get_external_origin(self):
         return self._handle.get_external_origin()
 
