@@ -1,13 +1,13 @@
 import datetime
-import sys
 import threading
-from abc import abstractmethod, abstractproperty
+from abc import abstractmethod
 from contextlib import AbstractContextManager
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union, cast
+from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Optional, Union, cast
 
-from dagster import check
+import dagster._check as check
 from dagster.api.get_server_id import sync_get_server_id
 from dagster.api.list_repositories import sync_list_repositories_grpc
+from dagster.api.notebook_data import sync_get_streaming_external_notebook_data_grpc
 from dagster.api.snapshot_execution_plan import sync_get_external_execution_plan_grpc
 from dagster.api.snapshot_partition import (
     sync_get_external_partition_config_grpc,
@@ -20,10 +20,7 @@ from dagster.api.snapshot_repository import sync_get_streaming_external_reposito
 from dagster.api.snapshot_schedule import sync_get_external_schedule_execution_data_grpc
 from dagster.api.snapshot_sensor import sync_get_external_sensor_execution_data_grpc
 from dagster.core.code_pointer import CodePointer
-from dagster.core.definitions.reconstructable import (
-    ReconstructablePipeline,
-    ReconstructableRepository,
-)
+from dagster.core.definitions.reconstruct import ReconstructablePipeline
 from dagster.core.errors import DagsterInvariantViolationError
 from dagster.core.execution.api import create_execution_plan
 from dagster.core.execution.plan.state import KnownExecutionState
@@ -46,11 +43,14 @@ from dagster.core.snap.execution_plan_snapshot import snapshot_from_execution_pl
 from dagster.grpc.impl import (
     get_external_schedule_execution,
     get_external_sensor_execution,
+    get_notebook_data,
     get_partition_config,
     get_partition_names,
     get_partition_set_execution_param_data,
     get_partition_tags,
 )
+from dagster.grpc.types import GetCurrentImageResult
+from dagster.serdes import deserialize_as
 from dagster.seven.compat.pendulum import PendulumDateTime
 from dagster.utils import merge_dicts
 from dagster.utils.hosted_user_process import external_repo_from_def
@@ -58,6 +58,8 @@ from dagster.utils.hosted_user_process import external_repo_from_def
 from .selector import PipelineSelector
 
 if TYPE_CHECKING:
+    from dagster.core.definitions.schedule_definition import ScheduleExecutionData
+    from dagster.core.definitions.sensor_definition import SensorExecutionData
     from dagster.core.host_representation import (
         ExternalPartitionConfigData,
         ExternalPartitionExecutionErrorData,
@@ -66,11 +68,7 @@ if TYPE_CHECKING:
         ExternalPartitionTagsData,
         ExternalScheduleExecutionErrorData,
     )
-    from dagster.core.definitions.schedule import ScheduleExecutionData
-    from dagster.core.definitions.sensor import SensorExecutionData
-    from dagster.core.host_representation.external_data import (
-        ExternalSensorExecutionErrorData,
-    )
+    from dagster.core.host_representation.external_data import ExternalSensorExecutionErrorData
 
 
 class RepositoryLocation(AbstractContextManager):
@@ -116,18 +114,37 @@ class RepositoryLocation(AbstractContextManager):
     def get_external_execution_plan(
         self,
         external_pipeline: ExternalPipeline,
-        run_config: Dict[str, Any],
+        run_config: Mapping[str, object],
         mode: str,
         step_keys_to_execute: Optional[List[str]],
         known_state: Optional[KnownExecutionState],
+        instance: Optional[DagsterInstance] = None,
     ) -> ExternalExecutionPlan:
         pass
 
+    def get_external_pipeline(self, selector: PipelineSelector) -> ExternalPipeline:
+        """Return the ExternalPipeline for a specific pipeline. Subclasses only
+        need to implement get_subset_external_pipeline_result to handle the case where
+        a solid selection is specified, which requires access to the underlying PipelineDefinition
+        to generate the subsetted pipeline snapshot."""
+        if not selector.solid_selection and not selector.asset_selection:
+            return self.get_repository(selector.repository_name).get_full_external_pipeline(
+                selector.pipeline_name
+            )
+
+        repo_handle = self.get_repository(selector.repository_name).handle
+
+        return ExternalPipeline(
+            self.get_subset_external_pipeline_result(selector).external_pipeline_data, repo_handle
+        )
+
     @abstractmethod
     def get_subset_external_pipeline_result(
         self, selector: PipelineSelector
     ) -> ExternalPipelineSubsetResult:
-        pass
+        """Returns a snapshot about an ExternalPipeline with a solid selection, which requires
+        access to the underlying PipelineDefinition. Callsites should likely use
+        `get_external_pipeline` instead."""
 
     @abstractmethod
     def get_external_partition_config(
@@ -178,7 +195,12 @@ class RepositoryLocation(AbstractContextManager):
     ) -> Union["SensorExecutionData", "ExternalSensorExecutionErrorData"]:
         pass
 
-    @abstractproperty
+    @abstractmethod
+    def get_external_notebook_data(self, notebook_path: str) -> bytes:
+        pass
+
+    @property
+    @abstractmethod
     def is_reload_supported(self) -> bool:
         pass
 
@@ -191,7 +213,8 @@ class RepositoryLocation(AbstractContextManager):
     def cleanup(self) -> None:
         pass
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def origin(self) -> RepositoryLocationOrigin:
         pass
 
@@ -201,22 +224,34 @@ class RepositoryLocation(AbstractContextManager):
             ({"image": self.container_image} if self.container_image else {}),
         )
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def executable_path(self) -> Optional[str]:
         pass
 
-    @abstractproperty
-    def container_image(self) -> str:
+    @property
+    @abstractmethod
+    def container_image(self) -> Optional[str]:
         pass
 
-    @abstractproperty
+    @property
+    def container_context(self) -> Optional[Dict[str, Any]]:
+        return None
+
+    @property
+    @abstractmethod
+    def entry_point(self) -> Optional[List[str]]:
+        pass
+
+    @property
+    @abstractmethod
     def repository_code_pointer_dict(self) -> Dict[str, CodePointer]:
         pass
 
     def get_repository_python_origin(self, repository_name: str) -> "RepositoryPythonOrigin":
         if repository_name not in self.repository_code_pointer_dict:
             raise DagsterInvariantViolationError(
-                "Unable to find repository name {} on GRPC server.".format(repository_name)
+                "Unable to find repository {}.".format(repository_name)
             )
 
         code_pointer = self.repository_code_pointer_dict[repository_name]
@@ -224,27 +259,39 @@ class RepositoryLocation(AbstractContextManager):
             executable_path=self.executable_path,
             code_pointer=code_pointer,
             container_image=self.container_image,
+            entry_point=self.entry_point,
+            container_context=self.container_context,
         )
 
 
 class InProcessRepositoryLocation(RepositoryLocation):
     def __init__(self, origin: InProcessRepositoryLocationOrigin):
+        from dagster.grpc.server import LoadedRepositories
+
         self._origin = check.inst_param(origin, "origin", InProcessRepositoryLocationOrigin)
 
-        self._recon_repo = self._origin.recon_repo
-
-        repo_def = self._recon_repo.get_definition()
-        pointer = self._recon_repo.pointer
-
-        self._repository_code_pointer_dict = {repo_def.name: pointer}
-
-        def_name = repo_def.name
-
-        self._external_repo = external_repo_from_def(
-            repo_def,
-            RepositoryHandle(repository_name=def_name, repository_location=self),
+        loadable_target_origin = self._origin.loadable_target_origin
+        self._loaded_repositories = LoadedRepositories(
+            loadable_target_origin,
+            self._origin.entry_point,
+            self._origin.container_image,
         )
-        self._repositories = {self._external_repo.name: self._external_repo}
+
+        self._repository_code_pointer_dict = self._loaded_repositories.code_pointers_by_repo_name
+
+        self._recon_repos = {
+            repo_name: self._loaded_repositories.get_recon_repo(repo_name)
+            for repo_name in self._repository_code_pointer_dict
+        }
+
+        self._repositories = {}
+        for repo_name in self._repository_code_pointer_dict:
+            recon_repo = self._loaded_repositories.get_recon_repo(repo_name)
+            repo_def = recon_repo.get_definition()
+            self._repositories[repo_name] = external_repo_from_def(
+                repo_def,
+                RepositoryHandle(repository_name=repo_name, repository_location=self),
+            )
 
     @property
     def is_reload_supported(self) -> bool:
@@ -256,21 +303,26 @@ class InProcessRepositoryLocation(RepositoryLocation):
 
     @property
     def executable_path(self) -> Optional[str]:
-        return sys.executable
+        return self._origin.loadable_target_origin.executable_path
 
     @property
-    def container_image(self) -> str:
-        return self._recon_repo.container_image
+    def container_image(self) -> Optional[str]:
+        return self._origin.container_image
+
+    @property
+    def container_context(self) -> Optional[Dict[str, Any]]:
+        return self._origin.container_context
+
+    @property
+    def entry_point(self) -> Optional[List[str]]:
+        return self._origin.entry_point
 
     @property
     def repository_code_pointer_dict(self) -> Dict[str, CodePointer]:
         return self._repository_code_pointer_dict
 
     def get_reconstructable_pipeline(self, name: str) -> ReconstructablePipeline:
-        return self.get_reconstructable_repository().get_reconstructable_pipeline(name)
-
-    def get_reconstructable_repository(self) -> ReconstructableRepository:
-        return self.origin.recon_repo
+        return self._recon_repos[name].get_reconstructable_pipeline(name)
 
     def get_repository(self, name: str) -> ExternalRepository:
         return self._repositories[name]
@@ -295,39 +347,44 @@ class InProcessRepositoryLocation(RepositoryLocation):
         from dagster.grpc.impl import get_external_pipeline_subset_result
 
         return get_external_pipeline_subset_result(
-            self.get_reconstructable_pipeline(selector.pipeline_name), selector.solid_selection
+            self.get_reconstructable_pipeline(selector.pipeline_name),
+            selector.solid_selection,
+            selector.asset_selection,
         )
 
     def get_external_execution_plan(
         self,
         external_pipeline: ExternalPipeline,
-        run_config: Dict[str, Any],
+        run_config: Mapping[str, object],
         mode: str,
         step_keys_to_execute: Optional[List[str]],
         known_state: Optional[KnownExecutionState],
+        instance: Optional[DagsterInstance] = None,
     ) -> ExternalExecutionPlan:
         check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
         check.dict_param(run_config, "run_config")
         check.str_param(mode, "mode")
-        check.opt_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
+        check.opt_nullable_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
         check.opt_inst_param(known_state, "known_state", KnownExecutionState)
+        check.opt_inst_param(instance, "instance", DagsterInstance)
 
+        execution_plan = create_execution_plan(
+            pipeline=self.get_reconstructable_pipeline(
+                external_pipeline.name
+            ).subset_for_execution_from_existing_pipeline(
+                external_pipeline.solids_to_execute, external_pipeline.asset_selection
+            ),
+            run_config=run_config,
+            mode=mode,
+            step_keys_to_execute=step_keys_to_execute,
+            known_state=known_state,
+            instance_ref=instance.get_ref() if instance and instance.is_persistent else None,
+        )
         return ExternalExecutionPlan(
             execution_plan_snapshot=snapshot_from_execution_plan(
-                create_execution_plan(
-                    pipeline=self.get_reconstructable_pipeline(
-                        external_pipeline.name
-                    ).subset_for_execution_from_existing_pipeline(
-                        external_pipeline.solids_to_execute
-                    ),
-                    run_config=run_config,
-                    mode=mode,
-                    step_keys_to_execute=step_keys_to_execute,
-                    known_state=known_state,
-                ),
+                execution_plan,
                 external_pipeline.identifying_pipeline_snapshot_id,
-            ),
-            represented_pipeline=external_pipeline,
+            )
         )
 
     def get_external_partition_config(
@@ -338,7 +395,7 @@ class InProcessRepositoryLocation(RepositoryLocation):
         check.str_param(partition_name, "partition_name")
 
         return get_partition_config(
-            recon_repo=self._recon_repo,
+            recon_repo=self._recon_repos[repository_handle.repository_name],
             partition_set_name=partition_set_name,
             partition_name=partition_name,
         )
@@ -351,7 +408,7 @@ class InProcessRepositoryLocation(RepositoryLocation):
         check.str_param(partition_name, "partition_name")
 
         return get_partition_tags(
-            recon_repo=self._recon_repo,
+            recon_repo=self._recon_repos[repository_handle.repository_name],
             partition_set_name=partition_set_name,
             partition_name=partition_name,
         )
@@ -363,7 +420,7 @@ class InProcessRepositoryLocation(RepositoryLocation):
         check.str_param(partition_set_name, "partition_set_name")
 
         return get_partition_names(
-            recon_repo=self._recon_repo,
+            recon_repo=self._recon_repos[repository_handle.repository_name],
             partition_set_name=partition_set_name,
         )
 
@@ -380,7 +437,7 @@ class InProcessRepositoryLocation(RepositoryLocation):
         check.opt_inst_param(scheduled_execution_time, "scheduled_execution_time", PendulumDateTime)
 
         return get_external_schedule_execution(
-            self._recon_repo,
+            recon_repo=self._recon_repos[repository_handle.repository_name],
             instance_ref=instance.get_ref(),
             schedule_name=schedule_name,
             scheduled_execution_timestamp=scheduled_execution_time.timestamp()
@@ -401,7 +458,12 @@ class InProcessRepositoryLocation(RepositoryLocation):
         cursor: Optional[str],
     ) -> Union["SensorExecutionData", "ExternalSensorExecutionErrorData"]:
         return get_external_sensor_execution(
-            self._recon_repo, instance.get_ref(), name, last_completion_time, last_run_key, cursor
+            self._recon_repos[repository_handle.repository_name],
+            instance.get_ref(),
+            name,
+            last_completion_time,
+            last_run_key,
+            cursor,
         )
 
     def get_external_partition_set_execution_param_data(
@@ -415,11 +477,15 @@ class InProcessRepositoryLocation(RepositoryLocation):
         check.list_param(partition_names, "partition_names", of_type=str)
 
         return get_partition_set_execution_param_data(
-            self._recon_repo,
+            recon_repo=self._recon_repos[repository_handle.repository_name],
             partition_set_name=partition_set_name,
             partition_names=partition_names,
         )
 
+    def get_external_notebook_data(self, notebook_path: str) -> bytes:
+        check.str_param(notebook_path, "notebook_path")
+        return get_notebook_data(notebook_path)
+
 
 class GrpcServerRepositoryLocation(RepositoryLocation):
     def __init__(
@@ -466,7 +532,9 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
 
         self._executable_path = None
         self._container_image = None
+        self._container_context = None
         self._repository_code_pointer_dict = None
+        self._entry_point = None
 
         try:
             self.client = DagsterGrpcClient(
@@ -500,8 +568,14 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
             self._repository_code_pointer_dict = (
                 list_repositories_response.repository_code_pointer_dict
             )
+            self._entry_point = list_repositories_response.entry_point
 
-            self._container_image = self._reload_current_image()
+            self._container_image = (
+                list_repositories_response.container_image
+                or self._reload_current_image()  # Back-compat for older gRPC servers that did not include container_image in ListRepositoriesResponse
+            )
+
+            self._container_context = list_repositories_response.container_context
 
             self._external_repositories_data = sync_get_streaming_external_repositories_data_grpc(
                 self.client,
@@ -530,6 +604,10 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
     def container_image(self) -> str:
         return cast(str, self._container_image)
 
+    @property
+    def container_context(self) -> Optional[Dict[str, Any]]:
+        return self._container_context
+
     @property
     def repository_code_pointer_dict(self) -> Dict[str, CodePointer]:
         return cast(Dict[str, CodePointer], self._repository_code_pointer_dict)
@@ -538,6 +616,10 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
     def executable_path(self) -> Optional[str]:
         return self._executable_path
 
+    @property
+    def entry_point(self) -> Optional[List[str]]:
+        return self._entry_point
+
     @property
     def port(self) -> Optional[int]:
         return self._port
@@ -554,8 +636,11 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
     def use_ssl(self) -> bool:
         return self._use_ssl
 
-    def _reload_current_image(self) -> str:
-        return self.client.get_current_image().current_image
+    def _reload_current_image(self) -> Optional[str]:
+        return deserialize_as(
+            self.client.get_current_image(),
+            GetCurrentImageResult,
+        ).current_image
 
     def cleanup(self) -> None:
         if self._heartbeat_shutdown_event:
@@ -591,16 +676,18 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
     def get_external_execution_plan(
         self,
         external_pipeline: ExternalPipeline,
-        run_config: Dict[str, Any],
+        run_config: Mapping[str, Any],
         mode: str,
         step_keys_to_execute: Optional[List[str]],
         known_state: Optional[KnownExecutionState],
+        instance: Optional[DagsterInstance] = None,
     ) -> ExternalExecutionPlan:
         check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
-        check.dict_param(run_config, "run_config")
+        run_config = check.dict_param(run_config, "run_config")
         check.str_param(mode, "mode")
-        check.opt_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
+        check.opt_nullable_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
         check.opt_inst_param(known_state, "known_state", KnownExecutionState)
+        check.opt_inst_param(instance, "instance", DagsterInstance)
 
         execution_plan_snapshot_or_error = sync_get_external_execution_plan_grpc(
             api_client=self.client,
@@ -608,15 +695,14 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
             run_config=run_config,
             mode=mode,
             pipeline_snapshot_id=external_pipeline.identifying_pipeline_snapshot_id,
+            asset_selection=external_pipeline.asset_selection,
             solid_selection=external_pipeline.solid_selection,
             step_keys_to_execute=step_keys_to_execute,
             known_state=known_state,
+            instance=instance,
         )
 
-        return ExternalExecutionPlan(
-            execution_plan_snapshot=execution_plan_snapshot_or_error,
-            represented_pipeline=external_pipeline,
-        )
+        return ExternalExecutionPlan(execution_plan_snapshot=execution_plan_snapshot_or_error)
 
     def get_subset_external_pipeline_result(
         self, selector: PipelineSelector
@@ -632,7 +718,10 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
         external_repository = self.get_repository(selector.repository_name)
         pipeline_handle = PipelineHandle(selector.pipeline_name, external_repository.handle)
         return sync_get_external_pipeline_subset_grpc(
-            self.client, pipeline_handle.get_external_origin(), selector.solid_selection
+            self.client,
+            pipeline_handle.get_external_origin(),
+            selector.solid_selection,
+            selector.asset_selection,
         )
 
     def get_external_partition_config(
@@ -719,3 +808,7 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
         return sync_get_external_partition_set_execution_param_data_grpc(
             self.client, repository_handle, partition_set_name, partition_names
         )
+
+    def get_external_notebook_data(self, notebook_path: str) -> bytes:
+        check.str_param(notebook_path, "notebook_path")
+        return sync_get_streaming_external_notebook_data_grpc(self.client, notebook_path)
