@@ -1,4 +1,4 @@
-from dagster import check
+import dagster._check as check
 from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus
 from dagster.serdes import ConfigurableClass, ConfigurableClassData
 
@@ -30,10 +30,10 @@ class DefaultRunCoordinator(RunCoordinator, ConfigurableClass):
         check.invariant(pipeline_run.status == PipelineRunStatus.NOT_STARTED)
 
         self._instance.launch_run(pipeline_run.run_id, context.workspace)
-        return self._instance.get_run_by_id(pipeline_run.run_id)
-
-    def can_cancel_run(self, run_id):
-        return self._instance.run_launcher.can_terminate(run_id)
+        run = self._instance.get_run_by_id(pipeline_run.run_id)
+        if run is None:
+            check.failed(f"Failed to reload run {pipeline_run.run_id}")
+        return run
 
     def cancel_run(self, run_id):
         return self._instance.run_launcher.terminate(run_id)
