@@ -1,6 +1,6 @@
-from collections import namedtuple
+from typing import List, NamedTuple, Optional
 
-from dagster import check
+import dagster._check as check
 from dagster.serdes import whitelist_for_serdes
 from dagster.utils.error import SerializableErrorInfo
 
@@ -11,16 +11,20 @@ class ScheduledExecutionResult:
 
 @whitelist_for_serdes
 class ScheduledExecutionSkipped(
-    namedtuple("_ScheduledExecutionSkipped", ""), ScheduledExecutionResult
+    NamedTuple("_ScheduledExecutionSkipped", []), ScheduledExecutionResult
 ):
     pass
 
 
 @whitelist_for_serdes
 class ScheduledExecutionFailed(
-    namedtuple("_ScheduledExecutionFailed", "run_id errors"), ScheduledExecutionResult
+    NamedTuple(
+        "_ScheduledExecutionFailed",
+        [("run_id", Optional[str]), ("errors", List[SerializableErrorInfo])],
+    ),
+    ScheduledExecutionResult,
 ):
-    def __new__(cls, run_id, errors):
+    def __new__(cls, run_id: Optional[str], errors: List[SerializableErrorInfo]):
         return super(ScheduledExecutionFailed, cls).__new__(
             cls,
             run_id=check.opt_str_param(run_id, "run_id"),
@@ -30,9 +34,9 @@ class ScheduledExecutionFailed(
 
 @whitelist_for_serdes
 class ScheduledExecutionSuccess(
-    namedtuple("_ScheduledExecutionSuccess", "run_id"), ScheduledExecutionResult
+    NamedTuple("_ScheduledExecutionSuccess", [("run_id", str)]), ScheduledExecutionResult
 ):
-    def __new__(cls, run_id):
+    def __new__(cls, run_id: str):
         return super(ScheduledExecutionSuccess, cls).__new__(
             cls, run_id=check.str_param(run_id, "run_id")
         )
