@@ -1,14 +1,17 @@
 import os
 import pickle
+from typing import Union
 
-from dagster import check
+import dagster._check as check
 from dagster.config import Field
 from dagster.config.source import StringSource
-from dagster.core.definitions.event_metadata import EventMetadataEntry
 from dagster.core.definitions.events import AssetKey, AssetMaterialization
+from dagster.core.definitions.metadata import MetadataEntry, MetadataValue
+from dagster.core.errors import DagsterInvariantViolationError
 from dagster.core.execution.context.input import InputContext
 from dagster.core.execution.context.output import OutputContext
 from dagster.core.storage.io_manager import IOManager, io_manager
+from dagster.core.storage.memoizable_io_manager import MemoizableIOManager
 from dagster.utils import PICKLE_PROTOCOL, mkdir_p
 from dagster.utils.backcompat import experimental
 
@@ -22,54 +25,64 @@ def fs_io_manager(init_context):
     your dagster.yaml file (which will be a temporary directory if not explicitly set).
 
     Serializes and deserializes output values using pickling and automatically constructs
-    the filepaths for the assets.
+    the filepaths for ops and assets.
+
+    Assigns each op output to a unique filepath containing run ID, step key, and output name.
+    Assigns each asset to a single filesystem path, at "<base_dir>/<asset_key>". If the asset key
+    has multiple components, the final component is used as the name of the file, and the preceding
+    components as parent directories under the base_dir.
+
+    Subsequent materializations of an asset will overwrite previous materializations of that asset.
+    So, with a base directory of "/my/base/path", an asset with key
+    `AssetKey(["one", "two", "three"])` would be stored in a file called "three" in a directory
+    with path "/my/base/path/one/two/".
 
     Example usage:
 
-    1. Specify a pipeline-level IO manager using the reserved resource key ``"io_manager"``,
-    which will set the given IO manager on all solids across a pipeline.
+    1. Specify a job-level IO manager using the reserved resource key ``"io_manager"``,
+    which will set the given IO manager on all ops in a job.
 
     .. code-block:: python
 
-        @solid
-        def solid_a(context, df):
+        from dagster import fs_io_manager, job, op
+
+        @op
+        def op_a():
+            # create df ...
             return df
 
-        @solid
-        def solid_b(context, df):
+        @op
+        def op_b(df):
             return df[:5]
 
-        @pipeline(
-            mode_defs=[
-                ModeDefinition(
-                    resource_defs={
-                        "io_manager": fs_io_manager.configured({"base_path": "/my/base/path"})
-                    }
-                )
-            ]
+        @job(
+            resource_defs={
+                "io_manager": fs_io_manager.configured({"base_dir": "/my/base/path"})
+            }
         )
-        def pipe():
-            solid_b(solid_a())
+        def job():
+            op_b(op_a())
 
 
-    2. Specify IO manager on :py:class:`OutputDefinition`, which allows the user to set
-    different IO managers on different step outputs.
+    2. Specify IO manager on :py:class:`Out`, which allows the user to set different IO managers on
+    different step outputs.
 
     .. code-block:: python
 
-        @solid(output_defs=[OutputDefinition(io_manager_key="my_io_manager")])
-        def solid_a(context, df):
+        from dagster import fs_io_manager, job, op, Out
+
+        @op(out=Out(io_manager_key="my_io_manager"))
+        def op_a():
+            # create df ...
             return df
 
-        @solid
-        def solid_b(context, df):
+        @op
+        def op_b(df):
             return df[:5]
 
-        @pipeline(
-            mode_defs=[ModeDefinition(resource_defs={"my_io_manager": fs_io_manager})]
-        )
-        def pipe():
-            solid_b(solid_a())
+        @job(resource_defs={"my_io_manager": fs_io_manager})
+        def job():
+            op_b(op_a())
 
     """
     base_dir = init_context.resource_config.get(
@@ -79,7 +92,7 @@ def fs_io_manager(init_context):
     return PickledObjectFilesystemIOManager(base_dir=base_dir)
 
 
-class PickledObjectFilesystemIOManager(IOManager):
+class PickledObjectFilesystemIOManager(MemoizableIOManager):
     """Built-in filesystem IO manager that stores and retrieves values using pickling.
 
     Args:
@@ -92,11 +105,19 @@ class PickledObjectFilesystemIOManager(IOManager):
         self.write_mode = "wb"
         self.read_mode = "rb"
 
-    def _get_path(self, context):
+    def _get_path(self, context: Union[InputContext, OutputContext]) -> str:
         """Automatically construct filepath."""
-        keys = context.get_run_scoped_output_identifier()
+        if context.has_asset_key:
+            path = context.get_asset_identifier()
+        else:
+            path = context.get_identifier()
 
-        return os.path.join(self.base_dir, *keys)
+        return os.path.join(self.base_dir, *path)
+
+    def has_output(self, context):
+        filepath = self._get_path(context)
+
+        return os.path.exists(filepath)
 
     def handle_output(self, context, obj):
         """Pickle the data and store the object to a file.
@@ -107,20 +128,42 @@ class PickledObjectFilesystemIOManager(IOManager):
         check.inst_param(context, "context", OutputContext)
 
         filepath = self._get_path(context)
-        context.log.debug(f"Writing file at: {filepath}")
 
         # Ensure path exists
         mkdir_p(os.path.dirname(filepath))
 
         with open(filepath, self.write_mode) as write_obj:
-            pickle.dump(obj, write_obj, PICKLE_PROTOCOL)
+            try:
+                pickle.dump(obj, write_obj, PICKLE_PROTOCOL)
+            except (AttributeError, RecursionError, ImportError, pickle.PicklingError) as e:
+                executor = context.step_context.pipeline_def.mode_definitions[0].executor_defs[0]
+
+                if isinstance(e, RecursionError):
+                    # if obj can't be pickled because of RecursionError then __str__() will also
+                    # throw a RecursionError
+                    obj_repr = f"{obj.__class__} exceeds recursion limit and"
+                else:
+                    obj_repr = obj.__str__()
+
+                raise DagsterInvariantViolationError(
+                    f"Object {obj_repr} is not picklable. You are currently using the "
+                    f"fs_io_manager and the {executor.name}. You will need to use a different "
+                    "io manager to continue using this output. For example, you can use the "
+                    "mem_io_manager with the in_process_executor.\n"
+                    "For more information on io managers, visit "
+                    "https://docs.dagster.io/concepts/io-management/io-managers \n"
+                    "For more information on executors, vist "
+                    "https://docs.dagster.io/deployment/executors#overview"
+                ) from e
+
+        context.add_output_metadata({"path": MetadataValue.path(os.path.abspath(filepath))})
 
     def load_input(self, context):
         """Unpickle the file and Load it to a data object."""
         check.inst_param(context, "context", InputContext)
 
-        filepath = self._get_path(context.upstream_output)
-        context.log.debug(f"Loading file from: {filepath}")
+        filepath = self._get_path(context)
+        context.add_input_metadata({"path": MetadataValue.path(os.path.abspath(filepath))})
 
         with open(filepath, self.read_mode) as read_obj:
             return pickle.load(read_obj)
@@ -164,7 +207,9 @@ class CustomPathPickledObjectFilesystemIOManager(IOManager):
 
         return AssetMaterialization(
             asset_key=AssetKey([context.pipeline_name, context.step_key, context.name]),
-            metadata_entries=[EventMetadataEntry.fspath(os.path.abspath(filepath))],
+            metadata_entries=[
+                MetadataEntry("path", value=MetadataValue.path(os.path.abspath(filepath)))
+            ],
         )
 
     def load_input(self, context):
@@ -192,24 +237,18 @@ def custom_path_fs_io_manager(init_context):
 
     .. code-block:: python
 
-        @solid(
-            output_defs=[
-                OutputDefinition(
-                    io_manager_key="io_manager", metadata={"path": "path/to/sample_output"}
-                )
-            ]
-        )
-        def sample_data(context, df):
+        from dagster import custom_path_fs_io_manager, job, op
+
+        @op(out=Out(metadata={"path": "path/to/sample_output"}))
+        def sample_data(df):
             return df[:5]
 
         my_custom_path_fs_io_manager = custom_path_fs_io_manager.configured(
             {"base_dir": "path/to/basedir"}
         )
 
-        @pipeline(
-            mode_defs=[ModeDefinition(resource_defs={"io_manager": my_custom_path_fs_io_manager})],
-        )
-        def pipe():
+        @job(resource_defs={"io_manager": my_custom_path_fs_io_manager})
+        def my_job():
             sample_data()
 
     """
