@@ -1,11 +1,14 @@
 """System-provided config objects and constructors."""
-from typing import AbstractSet, Any, Dict, List, NamedTuple, Optional, Type, Union, cast
+from typing import AbstractSet, Any, Dict, List, Mapping, NamedTuple, Optional, Type, Union, cast
 
-from dagster import check
+import dagster._check as check
 from dagster.core.definitions.configurable import ConfigurableDefinition
-from dagster.core.definitions.executor import ExecutorDefinition, execute_in_process_executor
-from dagster.core.definitions.pipeline import PipelineDefinition
-from dagster.core.definitions.resource import ResourceDefinition
+from dagster.core.definitions.executor_definition import (
+    ExecutorDefinition,
+    execute_in_process_executor,
+)
+from dagster.core.definitions.pipeline_definition import PipelineDefinition
+from dagster.core.definitions.resource_definition import ResourceDefinition
 from dagster.core.errors import DagsterInvalidConfigError
 from dagster.utils import ensure_single_item
 
@@ -54,7 +57,7 @@ class OutputsConfig(NamedTuple):
         elif isinstance(self.config, dict):
             return self.config.keys()
         else:
-            return {}
+            return set()
 
     @property
     def type_materializer_specs(self) -> list:
@@ -90,6 +93,7 @@ class ResolvedRunConfig(
             ("loggers", Dict[str, dict]),
             ("original_config_dict", Any),
             ("mode", str),
+            ("inputs", Dict[str, Any]),
         ],
     )
 ):
@@ -101,11 +105,13 @@ class ResolvedRunConfig(
         loggers=None,
         original_config_dict=None,
         mode=None,
+        inputs=None,
     ):
         check.opt_inst_param(execution, "execution", ExecutionConfig)
         check.opt_dict_param(original_config_dict, "original_config_dict")
         check.opt_dict_param(resources, "resources", key_type=str)
         check.opt_str_param(mode, "mode")
+        check.opt_dict_param(inputs, "inputs", key_type=str)
 
         if execution is None:
             execution = ExecutionConfig(None, None)
@@ -118,12 +124,13 @@ class ResolvedRunConfig(
             loggers=check.opt_dict_param(loggers, "loggers", key_type=str, value_type=dict),
             original_config_dict=original_config_dict,
             mode=mode,
+            inputs=inputs,
         )
 
     @staticmethod
     def build(
         pipeline_def: PipelineDefinition,
-        run_config: Optional[Dict[str, Any]] = None,
+        run_config: Optional[Mapping[str, object]] = None,
         mode: Optional[str] = None,
     ) -> "ResolvedRunConfig":
         """This method validates a given run config against the pipeline config schema. If
@@ -132,10 +139,11 @@ class ResolvedRunConfig(
         In case the run_config is invalid, this method raises a DagsterInvalidConfigError
         """
         from dagster.config.validate import process_config
+
         from .composite_descent import composite_descent
 
         check.inst_param(pipeline_def, "pipeline_def", PipelineDefinition)
-        run_config = check.opt_dict_param(run_config, "run_config")
+        run_config = check.opt_mapping_param(run_config, "run_config")
         check.opt_str_param(mode, "mode")
 
         mode = mode or pipeline_def.get_default_mode_name()
@@ -155,7 +163,7 @@ class ResolvedRunConfig(
                 run_config,
             )
 
-        config_value = config_evr.value
+        config_value = cast(Dict[str, Any], config_evr.value)
 
         mode_def = pipeline_def.get_mode_definition(mode)
 
@@ -191,6 +199,7 @@ class ResolvedRunConfig(
         solid_config_dict = composite_descent(
             pipeline_def, config_value.get(node_key, {}), mode_def.resource_defs
         )
+        input_configs = config_value.get("inputs", {})
 
         return ResolvedRunConfig(
             solids=solid_config_dict,
@@ -199,6 +208,7 @@ class ResolvedRunConfig(
             original_config_dict=run_config,
             resources=config_mapped_resource_configs,
             mode=mode,
+            inputs=input_configs,
         )
 
     def to_dict(self) -> Dict[str, Any]:
@@ -247,7 +257,7 @@ def config_map_executor(
 
 
 def config_map_resources(
-    resource_defs: Dict[str, ResourceDefinition],
+    resource_defs: Mapping[str, ResourceDefinition],
     resource_configs: Dict[str, Any],
 ) -> Dict[str, ResourceConfig]:
     """This function executes the config mappings for resources with respect to ConfigurableDefinition.
