@@ -1,25 +1,32 @@
+import logging
 import sys
 import threading
 import time
 import uuid
 from contextlib import ExitStack, contextmanager
+from typing import Callable, ContextManager, Iterator, Sequence, Tuple
 
 import pendulum
-from dagster import check
-from dagster.core.definitions.sensor import DEFAULT_SENSOR_DAEMON_INTERVAL
+
+import dagster._check as check
 from dagster.core.host_representation.grpc_server_registry import ProcessGrpcServerRegistry
 from dagster.core.instance import DagsterInstance
-from dagster.core.workspace.dynamic_workspace import DynamicWorkspace
+from dagster.core.workspace import IWorkspace
+from dagster.core.workspace.load_target import WorkspaceLoadTarget
+from dagster.daemon.auto_run_reexecution.event_log_consumer import EventLogConsumerDaemon
 from dagster.daemon.daemon import (
     BackfillDaemon,
     DagsterDaemon,
+    MonitoringDaemon,
     SchedulerDaemon,
     SensorDaemon,
-    get_default_daemon_logger,
 )
 from dagster.daemon.run_coordinator.queued_run_coordinator_daemon import QueuedRunCoordinatorDaemon
 from dagster.daemon.types import DaemonHeartbeat, DaemonStatus
 from dagster.utils.interrupts import raise_interrupts_as
+from dagster.utils.log import configure_loggers
+
+from .workspace import DaemonWorkspace
 
 # How long beyond the expected heartbeat will the daemon be considered healthy
 DEFAULT_DAEMON_HEARTBEAT_TOLERANCE_SECONDS = 300
@@ -33,12 +40,15 @@ DEFAULT_HEARTBEAT_INTERVAL_SECONDS = 30
 # How long after an error is raised in a daemon that it's still included in the status for that daemon
 DEFAULT_DAEMON_ERROR_INTERVAL_SECONDS = 300
 
-
 THREAD_CHECK_INTERVAL = 5
 
 HEARTBEAT_CHECK_INTERVAL = 15
 
 
+DAEMON_GRPC_SERVER_RELOAD_INTERVAL = 60
+DAEMON_GRPC_SERVER_HEARTBEAT_TTL = 120
+
+
 def _sorted_quoted(strings):
     return "[" + ", ".join(["'{}'".format(s) for s in sorted(list(strings))]) + "]"
 
@@ -50,33 +60,50 @@ def create_daemons_from_instance(instance):
     ]
 
 
+def create_daemon_grpc_server_registry(instance):
+    return ProcessGrpcServerRegistry(
+        reload_interval=DAEMON_GRPC_SERVER_RELOAD_INTERVAL,
+        heartbeat_ttl=DAEMON_GRPC_SERVER_HEARTBEAT_TTL,
+        startup_timeout=instance.code_server_process_startup_timeout,
+    )
+
+
 @contextmanager
 def daemon_controller_from_instance(
-    instance,
-    heartbeat_interval_seconds=DEFAULT_HEARTBEAT_INTERVAL_SECONDS,
-    heartbeat_tolerance_seconds=DEFAULT_DAEMON_HEARTBEAT_TOLERANCE_SECONDS,
-    wait_for_processes_on_exit=False,
-    gen_daemons=create_daemons_from_instance,
-    error_interval_seconds=DEFAULT_DAEMON_ERROR_INTERVAL_SECONDS,
+    instance: DagsterInstance,
+    workspace_load_target: WorkspaceLoadTarget,
+    heartbeat_interval_seconds: int = DEFAULT_HEARTBEAT_INTERVAL_SECONDS,
+    heartbeat_tolerance_seconds: int = DEFAULT_DAEMON_HEARTBEAT_TOLERANCE_SECONDS,
+    wait_for_processes_on_exit: bool = False,
+    gen_daemons: Callable[
+        [DagsterInstance], Iterator[DagsterDaemon]
+    ] = create_daemons_from_instance,
+    error_interval_seconds: int = DEFAULT_DAEMON_ERROR_INTERVAL_SECONDS,
 ):
     check.inst_param(instance, "instance", DagsterInstance)
+    check.inst_param(workspace_load_target, "workspace_load_target", WorkspaceLoadTarget)
     grpc_server_registry = None
-
     try:
         with ExitStack() as stack:
-            grpc_server_registry = stack.enter_context(ProcessGrpcServerRegistry())
+            grpc_server_registry = stack.enter_context(create_daemon_grpc_server_registry(instance))
             daemons = [stack.enter_context(daemon) for daemon in gen_daemons(instance)]
 
+            instance_ref = instance.get_ref()
+
             # Create this in each daemon to generate a workspace per-daemon
             @contextmanager
-            def gen_workspace(_instance):
-                with DynamicWorkspace(grpc_server_registry) as workspace:
-                    yield workspace
+            def _context_fn():
+                with DaemonWorkspace(
+                    grpc_server_registry, workspace_load_target
+                ) as workspace, DagsterInstance.from_ref(  # clone instance object so each thread has its own
+                    instance_ref
+                ) as thread_instance:
+                    yield thread_instance, workspace
 
             with DagsterDaemonController(
                 instance,
                 daemons,
-                gen_workspace,
+                _context_fn,
                 heartbeat_interval_seconds=heartbeat_interval_seconds,
                 heartbeat_tolerance_seconds=heartbeat_tolerance_seconds,
                 error_interval_seconds=error_interval_seconds,
@@ -90,12 +117,13 @@ def daemon_controller_from_instance(
 class DagsterDaemonController:
     def __init__(
         self,
-        instance,
-        daemons,
-        gen_workspace,
-        heartbeat_interval_seconds=DEFAULT_HEARTBEAT_INTERVAL_SECONDS,
-        heartbeat_tolerance_seconds=DEFAULT_DAEMON_HEARTBEAT_TOLERANCE_SECONDS,
-        error_interval_seconds=DEFAULT_DAEMON_ERROR_INTERVAL_SECONDS,
+        instance: DagsterInstance,
+        daemons: Sequence[DagsterDaemon],
+        context_fn: Callable[[], ContextManager[Tuple[DagsterInstance, IWorkspace]]],
+        heartbeat_interval_seconds: int = DEFAULT_HEARTBEAT_INTERVAL_SECONDS,
+        heartbeat_tolerance_seconds: int = DEFAULT_DAEMON_HEARTBEAT_TOLERANCE_SECONDS,
+        error_interval_seconds: int = DEFAULT_DAEMON_ERROR_INTERVAL_SECONDS,
+        handler: str = "default",
     ):
 
         self._daemon_uuid = str(uuid.uuid4())
@@ -109,7 +137,7 @@ class DagsterDaemonController:
             for daemon in check.list_param(daemons, "daemons", of_type=DagsterDaemon)
         }
 
-        self._gen_workspace = check.callable_param(gen_workspace, "gen_workspace")
+        self._context_fn = check.callable_param(context_fn, "context_fn")
 
         self._heartbeat_interval_seconds = check.numeric_param(
             heartbeat_interval_seconds, "heartbeat_interval_seconds"
@@ -124,22 +152,23 @@ class DagsterDaemonController:
 
         self._daemon_shutdown_event = threading.Event()
 
-        self._logger = get_default_daemon_logger("dagster-daemon")
+        configure_loggers(handler=handler)
+
+        self._logger = logging.getLogger("dagster.daemon")
         self._logger.info(
-            "instance is configured with the following daemons: {}".format(
-                _sorted_quoted(type(daemon).__name__ for daemon in self.daemons)
-            )
+            "instance is configured with the following daemons: %s",
+            _sorted_quoted(type(daemon).__name__ for daemon in self.daemons),
         )
 
         self._last_healthy_heartbeat_times = {}
 
         for daemon_type, daemon in self._daemons.items():
             self._daemon_threads[daemon_type] = threading.Thread(
-                target=daemon.run_loop,
+                target=daemon.run_daemon_loop,
                 args=(
                     self._daemon_uuid,
                     self._daemon_shutdown_event,
-                    gen_workspace,
+                    context_fn,
                     heartbeat_interval_seconds,
                     error_interval_seconds,
                 ),
@@ -158,31 +187,39 @@ class DagsterDaemonController:
         thread = self._daemon_threads[daemon_type]
         return thread.is_alive()
 
-    def _daemon_heartbeat_healthy(self, daemon_type):
+    def _daemon_heartbeat_health(self):
         now = time.time()
         try:
-            is_healthy = get_daemon_status(
+            daemon_statuses_by_type = get_daemon_statuses(
                 self._instance,
-                daemon_type,
+                daemon_types=self._daemons.keys(),
                 heartbeat_interval_seconds=self._heartbeat_interval_seconds,
                 heartbeat_tolerance_seconds=self._heartbeat_tolerance_seconds,
                 ignore_errors=True,
-            ).healthy
-            if is_healthy:
-                self._last_healthy_heartbeat_times[daemon_type] = now
-            return is_healthy
-        except Exception:  # pylint: disable=broad-except
+            )
+            daemon_health_by_type = {
+                daemon_type: daemon_status.healthy
+                for (daemon_type, daemon_status) in daemon_statuses_by_type.items()
+            }
+
+            for daemon_type, is_daemon_healthy in daemon_health_by_type.items():
+                if is_daemon_healthy:
+                    self._last_healthy_heartbeat_times[daemon_type] = now
+
+            return daemon_health_by_type
+        except Exception:
             self._logger.warning(
-                "Error attempting to check {daemon_type} heartbeat:".format(
-                    daemon_type=daemon_type,
-                ),
+                "Error attempting to check daemon heartbeats",
                 exc_info=sys.exc_info,
             )
 
-            return (
-                self._last_healthy_heartbeat_times[daemon_type]
-                > now - self._heartbeat_tolerance_seconds
-            )
+            return {
+                daemon_type: (
+                    self._last_healthy_heartbeat_times[daemon_type]
+                    > now - self._heartbeat_tolerance_seconds
+                )
+                for daemon_type in self._daemons.keys()
+            }
 
     def check_daemon_threads(self):
         failed_daemons = [
@@ -201,8 +238,8 @@ class DagsterDaemonController:
     def check_daemon_heartbeats(self):
         failed_daemons = [
             daemon_type
-            for daemon_type in self._daemon_threads
-            if not self._daemon_heartbeat_healthy(daemon_type)
+            for daemon_type, is_daemon_healthy in self._daemon_heartbeat_health().items()
+            if not is_daemon_healthy
         ]
 
         if failed_daemons:
@@ -243,11 +280,7 @@ class DagsterDaemonController:
                 thread.join(timeout=30)
 
                 if thread.is_alive():
-                    self._logger.error(
-                        "Thread for {daemon_type} did not shut down gracefully".format(
-                            daemon_type=daemon_type
-                        )
-                    )
+                    self._logger.error("Thread for %s did not shut down gracefully", daemon_type)
 
     def _add_daemon(self, daemon):
         self._daemons[daemon.daemon_type()] = daemon
@@ -262,15 +295,19 @@ class DagsterDaemonController:
 
 def create_daemon_of_type(daemon_type, instance):
     if daemon_type == SchedulerDaemon.daemon_type():
-        return SchedulerDaemon(interval_seconds=DEFAULT_DAEMON_INTERVAL_SECONDS)
+        return SchedulerDaemon()
     elif daemon_type == SensorDaemon.daemon_type():
-        return SensorDaemon(interval_seconds=DEFAULT_SENSOR_DAEMON_INTERVAL)
+        return SensorDaemon()
     elif daemon_type == QueuedRunCoordinatorDaemon.daemon_type():
         return QueuedRunCoordinatorDaemon(
             interval_seconds=instance.run_coordinator.dequeue_interval_seconds
         )
     elif daemon_type == BackfillDaemon.daemon_type():
         return BackfillDaemon(interval_seconds=DEFAULT_DAEMON_INTERVAL_SECONDS)
+    elif daemon_type == MonitoringDaemon.daemon_type():
+        return MonitoringDaemon(interval_seconds=instance.run_monitoring_poll_interval_seconds)
+    elif daemon_type == EventLogConsumerDaemon.daemon_type():
+        return EventLogConsumerDaemon()
     else:
         raise Exception(f"Unexpected daemon type {daemon_type}")
 
@@ -283,20 +320,17 @@ def all_daemons_healthy(
 ):
     """
     True if all required daemons have had a recent heartbeat with no errors
-
     """
 
-    statuses = [
-        get_daemon_status(
-            instance,
-            daemon_type,
-            heartbeat_interval_seconds=heartbeat_interval_seconds,
-            heartbeat_tolerance_seconds=heartbeat_tolerance_seconds,
-            curr_time_seconds=curr_time_seconds,
-        )
-        for daemon_type in instance.get_required_daemon_types()
-    ]
-    return all([status.healthy for status in statuses])
+    statuses_by_type = get_daemon_statuses(
+        instance,
+        daemon_types=instance.get_required_daemon_types(),
+        heartbeat_interval_seconds=heartbeat_interval_seconds,
+        heartbeat_tolerance_seconds=heartbeat_tolerance_seconds,
+        curr_time_seconds=curr_time_seconds,
+    )
+
+    return all(status.healthy for status in statuses_by_type.values())
 
 
 def all_daemons_live(
@@ -309,23 +343,21 @@ def all_daemons_live(
     True if all required daemons have had a recent heartbeat, regardless of if it contained errors.
     """
 
-    statuses = [
-        get_daemon_status(
-            instance,
-            daemon_type,
-            heartbeat_interval_seconds=heartbeat_interval_seconds,
-            heartbeat_tolerance_seconds=heartbeat_tolerance_seconds,
-            curr_time_seconds=curr_time_seconds,
-            ignore_errors=True,
-        )
-        for daemon_type in instance.get_required_daemon_types()
-    ]
-    return all([status.healthy for status in statuses])
+    statuses_by_type = get_daemon_statuses(
+        instance,
+        daemon_types=instance.get_required_daemon_types(),
+        heartbeat_interval_seconds=heartbeat_interval_seconds,
+        heartbeat_tolerance_seconds=heartbeat_tolerance_seconds,
+        curr_time_seconds=curr_time_seconds,
+        ignore_errors=True,
+    )
+
+    return all(status.healthy for status in statuses_by_type.values())
 
 
-def get_daemon_status(
+def get_daemon_statuses(
     instance,
-    daemon_type,
+    daemon_types,
     curr_time_seconds=None,
     ignore_errors=False,
     heartbeat_interval_seconds=DEFAULT_HEARTBEAT_INTERVAL_SECONDS,
@@ -335,40 +367,45 @@ def get_daemon_status(
         curr_time_seconds, "curr_time_seconds", default=pendulum.now("UTC").float_timestamp
     )
 
-    # check if daemon required
-    if daemon_type not in instance.get_required_daemon_types():
-        return DaemonStatus(
-            daemon_type=daemon_type, required=False, healthy=None, last_heartbeat=None
-        )
-
-    # check if daemon present
+    daemon_statuses_by_type = {}
     heartbeats = instance.get_daemon_heartbeats()
-    if daemon_type not in heartbeats:
-        return DaemonStatus(
-            daemon_type=daemon_type, required=True, healthy=False, last_heartbeat=None
-        )
 
-    # check if daemon has sent a recent heartbeat
-    latest_heartbeat = heartbeats[daemon_type]
-    hearbeat_timestamp = latest_heartbeat.timestamp
-    maximum_tolerated_time = (
-        hearbeat_timestamp + heartbeat_interval_seconds + heartbeat_tolerance_seconds
-    )
-    healthy = curr_time_seconds <= maximum_tolerated_time
+    for daemon_type in daemon_types:
+        # check if daemon is not required
+        if daemon_type not in instance.get_required_daemon_types():
+            daemon_statuses_by_type[daemon_type] = DaemonStatus(
+                daemon_type=daemon_type, required=False, healthy=None, last_heartbeat=None
+            )
+        else:
+            # check if daemon has a heartbeat
+            if daemon_type not in heartbeats:
+                daemon_statuses_by_type[daemon_type] = DaemonStatus(
+                    daemon_type=daemon_type, required=True, healthy=False, last_heartbeat=None
+                )
+            else:
+                # check if daemon has sent a recent heartbeat
+                latest_heartbeat = heartbeats[daemon_type]
+                hearbeat_timestamp = latest_heartbeat.timestamp
+                maximum_tolerated_time = (
+                    hearbeat_timestamp + heartbeat_interval_seconds + heartbeat_tolerance_seconds
+                )
+                healthy = curr_time_seconds <= maximum_tolerated_time
 
-    if not ignore_errors and latest_heartbeat.errors:
-        healthy = False
+                if not ignore_errors and latest_heartbeat.errors:
+                    healthy = False
 
-    return DaemonStatus(
-        daemon_type=daemon_type,
-        required=True,
-        healthy=healthy,
-        last_heartbeat=heartbeats[daemon_type],
-    )
+                daemon_statuses_by_type[daemon_type] = DaemonStatus(
+                    daemon_type=daemon_type,
+                    required=True,
+                    healthy=healthy,
+                    last_heartbeat=heartbeats[daemon_type],
+                )
+
+    return daemon_statuses_by_type
 
 
 def debug_daemon_heartbeats(instance):
-    daemon = SensorDaemon(interval_seconds=DEFAULT_DAEMON_INTERVAL_SECONDS)
+    daemon = SensorDaemon()
     timestamp = pendulum.now("UTC").float_timestamp
     instance.add_daemon_heartbeat(DaemonHeartbeat(timestamp, daemon.daemon_type(), None, None))
     returned_timestamp = instance.get_daemon_heartbeats()[daemon.daemon_type()].timestamp
