@@ -1,18 +1,20 @@
-from dagster import check
-from dagster.core.host_representation import ExternalPipeline, RepositoryLocationHandle
+from typing import List, Optional
+
+import dagster._check as check
+from dagster.core.host_representation import RepositoryLocation
+from dagster.core.host_representation.external import ExternalPipeline
 from dagster.core.host_representation.origin import ExternalPipelineOrigin
 from dagster.core.host_representation.selector import PipelineSelector
 
 
-def external_pipeline_from_location_handle(
-    repository_location_handle, external_pipeline_origin, solid_selection
-):
-    check.inst_param(
-        repository_location_handle, "repository_location_handle", RepositoryLocationHandle
-    )
+def external_pipeline_from_location(
+    repo_location: RepositoryLocation,
+    external_pipeline_origin: ExternalPipelineOrigin,
+    solid_selection: Optional[List[str]],
+) -> ExternalPipeline:
+    check.inst_param(repo_location, "repository_location", RepositoryLocation)
     check.inst_param(external_pipeline_origin, "external_pipeline_origin", ExternalPipelineOrigin)
 
-    repo_location = repository_location_handle.create_location()
     repo_name = external_pipeline_origin.external_repository_origin.repository_name
     pipeline_name = external_pipeline_origin.pipeline_name
 
@@ -31,9 +33,4 @@ def external_pipeline_from_location_handle(
         solid_selection=solid_selection,
     )
 
-    subset_pipeline_result = repo_location.get_subset_external_pipeline_result(pipeline_selector)
-    external_pipeline = ExternalPipeline(
-        subset_pipeline_result.external_pipeline_data,
-        external_repo.handle,
-    )
-    return external_pipeline
+    return repo_location.get_external_pipeline(pipeline_selector)
